%% Anm class (Analysis model class)
%
% This is an abstract super-class in the Object Oriented Programming (OOP)
% paradigm that generically specifies an analysis model in the <main.html
% LESM (Linear Elements Structure Model)> program.
%
% This super-class is abstract because one cannot instantiate objects from
% it, since it contains abstract methods that should be implemented in
% (derived) sub-classes.
%
% Essentially, this super-class defines abstract methods that define
% the generic behavior of an analysis model in a linear element structure
% analysis program. These abstract methods are the functions that should
% be implemented in a (derived) sub-class that deals with specific types
% of analysis, such as a planar frame, planar truss, or a grillage model.
%
%% Sub-classes
% In the current implementation of LESM program there are three sub-classes
% of this super-class:
  %%%
% * <anm_truss2d.html Planar truss analysis>.
% * <anm_frame2d.html Planar frame analysis>.
% * <anm_grillage.html Grillage analysis model>.
%
%% Authors
% Luiz Fernando Martha and Rafael Lopez Rangel
%
%% History
% @version 1.00
%
% Initial version: August-December 2016
%%%
% Prepared for the course ENG 1240 - Analise de Estruturas III, 2016.2,
% Department of Civil Engineering, PUC-Rio.
%
classdef Anm < hgsetget
    %%
    % <http://www.mathworks.com/help/matlab/handle-classes.html
    % See documentation on *hgsetget* handle super-class>.

    %% Public attributes
    properties (SetAccess = public, GetAccess = public)
        analysis_type = 0;   % type of analysis model
        ndof = 0;            % number of degrees-of-freedom per node
    end

    %% Constructor method
    methods
        %-----------------------------------------------------------------------
        function obj = Anm(type,n)
            if (nargin > 0)
                obj.analysis_type = type;
                obj.ndof = n;
            end
        end
    end

    %% Public set methods
    methods
        %-----------------------------------------------------------------------
        % Set analysis type parameter value
        function obj = set.analysis_type(obj,type)
            obj.analysis_type = type;
        end
  
        %-----------------------------------------------------------------------
        % Set ndof parameter value
        function obj = set.ndof(obj,n)
            obj.ndof = n;
        end
    end

    %% Public get methods
    methods
        %-----------------------------------------------------------------------
        % Get analysis type parameter value
        function type = get.analysis_type(obj)
            type = obj.analysis_type;
        end

        %-----------------------------------------------------------------------
        % Get ndof parameter value
        function n = get.ndof(obj)
            n = obj.ndof;
        end
    end

    %% Abstract methods
    methods (Abstract)
        %-----------------------------------------------------------------------
        % Assemble rotation transformation matrix from global system to local system.
        % Output:
        %  rot:   rotation transformation matrix
        % Input arguments:
        %  elem:  handle to element object
        rot = gblToLocElemRotMtx(obj,elem)

        %-----------------------------------------------------------------------
        % Setup global d.o.f (degree of freedown) numbering ID matrix.
        % Free d.o.f.'s are numbered first.
        % Counts total number of equations of free d.o.f.'s and total number
        % of equations of fixed d.o.f.'s, and stores these numbers in global
        % variables.
        % Input arguments:
        %  drv: handle to analysis driver object
        obj = setupDOFNum(obj,drv)

        %-----------------------------------------------------------------------
        % Store prescribed displacements (known support settlement values) in global
        % displacement vector.
        % Input arguments:
        %  drv: handle to analysis driver object
        obj = setupPrescDispl(obj,drv)
        
        %-----------------------------------------------------------------------
        % Assemble member stiffness matrix in local system for a given element.
        % Output:
        %  kel:   target element stiffness matrix in local system
        % Input arguments:
        %  elem:  handle to element object
        kel = elemLocStiffMtx(obj,elem)

        %-----------------------------------------------------------------------
        % Add nodal load components to any term of the global forcing vector,
        % incluing the terms that correspond to constrained d.o.f.
        % Input arguments:
        %  drv: handle to analysis driver object
        obj = nodalLoads(obj,drv)

        %-----------------------------------------------------------------------
        % Assemble member fixed end force (FEF) vector in local system for a
        % uniformely distributed load applied to a given element according to
        % target analisys model.
        % Output:
        %  fel:   fixed end force vector in local system
        % Input arguments:
        %  elem:  handle to element object
        fel = elemLocUniformLoadFEF(obj,elem)

        %-----------------------------------------------------------------------
        % Assemble member fixed end force (FEF) vector in local system for a
        % linearly distributed load applied to a given element according to
        % target analisys model.
        % Output:
        %  fel:  fixed end force vector in local system
        % Input arguments:
        %  elem: handle to element object
        fel = elemLocLinearLoadFEF(obj,elem)

        %-----------------------------------------------------------------------
        % Assemble member fixed end force (FEF) vector in local system for a
        % thermal expansion of a given element according to target analisys model.
        % Output:
        %  fel:   fixed end force vector in local system
        % Input arguments:
        %  elem:  handle to element object
        fel = elemLocThermalLoadFEF(obj,elem)

        %-----------------------------------------------------------------------
        % Initialize given element internal force arrays with null values.
        % Input arguments:
        %  elem:   handle to element object
        obj = initIntForce(obj,elem)

        %-----------------------------------------------------------------------
        % Assemble contribution of given internal force vector (fel) to
        % arrays of internal forces of given element (elem), depending on
        % the current analysis model.
        % Input arguments:
        %  elem:   handle to element object
        %  fel: element internal force vector in local system
        obj = assembleIntForce(obj,elem,fel)
    end

    %% Public methods
    methods
        %-----------------------------------------------------------------------
        % Cleans data structure of a Anm object.
        function obj = clean(obj)
            obj.analysis_type = 0;
            obj.ndof = 0;
        end
    end
end