%% Anm_Frame2D class
%
% This is a sub-class, in the Object Oriented Programming (OOP) paradigm,
% of super-class <anm.html *Anm*> in the <main.html LESM (Linear Elements 
% Structure Model)> program. This sub-class implements abstract methods, 
% defined in super-class *Anm*, that deal with planar frame analysis model
% of linear structure elements.
%
%% Authors
% Luiz Fernando Martha and Rafael Lopez Rangel
%
%% Frame 2D analysis
% A frame model is also made up of bars, which are called beams
% (horizontal bars) or columns (vertical bars). Inclined members
% are also called beams. In the present context, these types of bars
% are generically called "frame members". A continuous beam (an
% assemblage of connected beams) is considered as a frame model by
% LESM.  These are the basic assumptions of a frame model:
  %%%
% * Frame members are usually rigidly connected at the joints.
%   However, a frame member might have a hinge (rotation liberation)
%   at an end or hinges at both ends. A frame member with hinges at
%   both end works like a truss member.  A truss model could be seen
%   a frame model with complete hinges at both its nodes.
% * A 2D frame model is laid on the XY plane.
% * By assumption, there is only in-plane behavior, that is, there is
%   no displacement transversal to the frame plane.
% * Internal forces at any cross-section of a plane frame member are:
%   axial force, shear force, and bending moment.
% * The geometric parameters of a plane frame member cross-section
%   are its full area (for axial behavior), its effective shear area,
%   and its moment of inertia (for bending behavior).
% * Each node of a 2D frame model has three d.o.f.'s: a horizontal
%   displacement in X direction, a vertical displacement in Y
%   direction, and a rotation about the Z axis.
%
%% History
% @version 1.00
%
% Initial version: August-December 2016
%%%
% Prepared for the course ENG 1240 - Anlise de Estruturas III, 2016.2,
% Department of Civil Engineering, PUC-Rio.
%
classdef Anm_Frame2D < Anm
    %%
    % <anm.html See documentation on *Anm* super-class>.

    %% Constructor method
    methods
        %-----------------------------------------------------------------------
        function obj = Anm_Frame2D()
            include_constants;
            obj = obj@Anm(FRAME2D_ANALYSIS,3);
        end
    end

    %% Public methods
    % Implementation of the abstract methods declared in super-class <anm.html *Anm*>.
    methods
        %-----------------------------------------------------------------------
        % Assemble rotation transformation matrix from global system to local system.
        % Output:
        %  rot: rotation transformation matrix
        % Input arguments:
        %  elem: handle to element object
        function rot = gblToLocElemRotMtx(~,elem)
            % Get element orientation angle cosine and sine
            cx = elem.cosine_X;            % element orientation cosine
            cy = elem.cosine_Y;            % element orientation sine

            % Assemble rotation transformation matrix
%   *** COMPLETE HERE - Anm_Frame2D-01 ***






%   *** COMPLETE HERE - Anm_Frame2D-01 ***
        end

        %-----------------------------------------------------------------------
        % Setup global d.o.f. (degree of freedown) numbering ID matrix.
        % Free d.o.f.'s are numbered first.
        % Counts total number of equations of free d.o.f.'s and total number
        % of equations of fixed d.o.f.'s, and stores these numbers in global
        % variables.
        % Input arguments:
        %  drv: handle to analysis driver object
        % Initialize ID matrix:
        %  if ID(k,n) = 0, d.o.f. k of node n is free
        %  if ID(k,n) = 1, d.o.f. k of node n is fixed
        function setupDOFNum(obj,drv)
            % Dimension global d.o.f. numbering matrix
            drv.ID = zeros(obj.ndof,drv.nnp);

            % Compute total number of fixed d.o.f.
            drv.neqfixed = 0;

            for n = 1:drv.nnp

                if drv.nodes(n).ebc(1) == 1
                    drv.neqfixed = drv.neqfixed + 1;
                    drv.ID(1,n) = 1;
                end

                if drv.nodes(n).ebc(2) == 1
                    drv.neqfixed = drv.neqfixed + 1;
                    drv.ID(2,n) = 1;
                end

                if drv.nodes(n).ebc(6) == 1
                    drv.neqfixed = drv.neqfixed + 1;
                    drv.ID(3,n) = 1;
                end

            end

            % Compute total number of free d.o.f.
            drv.neqfree = drv.neq - drv.neqfixed;
        end
        
        %-----------------------------------------------------------------------
        % Store prescribed displacements (known support settlement values) in global
        % displacement vector.
        % Avoid storing a prescribed displacement component in a position of
        % the global displacement vector that corresponds to a free d.o.f.
        % Input arguments:
        %  drv: handle to analysis driver object
        function setupPrescDispl(~,drv)
          for n = 1:drv.nnp
            if size(drv.nodes(n).prescDispl,2) > 0

              if (drv.ID(1,n) > drv.neqfree) && ...
                 (drv.nodes(n).prescDispl(1) ~= 0)
                drv.D(drv.ID(1,n)) = drv.nodes(n).prescDispl(1);
              end

              if (drv.ID(2,n) > drv.neqfree) && ...
                 (drv.nodes(n).prescDispl(2) ~= 0)
                drv.D(drv.ID(2,n)) = drv.nodes(n).prescDispl(2);
              end

              if (drv.ID(3,n) > drv.neqfree) && ...
                 (drv.nodes(n).prescDispl(6) ~= 0)
                drv.D(drv.ID(3,n)) = drv.nodes(n).prescDispl(6);
              end

            end
          end
        end

        %-----------------------------------------------------------------------
        % Assemble member stiffness matrix in local system for a given element.
        % Output:
        %  kel: target element stiffness matrix in local system
        % Input arguments:
        %  elem: handle to element object
        function kel = elemLocStiffMtx(~,elem)
            % Assemble element stiffness matrix in local system
            % Compute axial stiffness coefficients for given member
            kea = elem.axialStiffCoeff();

            % Compute flexural stiffness coefficients for given member
            kef = elem.flexuralStiffCoeff_XY();

%   *** COMPLETE HERE - Anm_Frame2D-02 ***






%   *** COMPLETE HERE - Anm_Frame2D-02 ***
        end

        %-----------------------------------------------------------------------
        % Add nodal load components to any term of the global forcing vector,
        % incluing the terms that correspond to constrained d.o.f.
        % Input arguments:
        %  drv: handle to analysis driver object
        function nodalLoads(~,drv)
          for n = 1:drv.nnp
            if size(drv.nodes(n).nodalLoad,2) > 0

              id = drv.ID(1,n);
              drv.F(id) = drv.F(id) + drv.nodes(n).nodalLoad(1);

              id = drv.ID(2,n);
              drv.F(id) = drv.F(id) + drv.nodes(n).nodalLoad(2);

              id = drv.ID(3,n);
              drv.F(id) = drv.F(id) + drv.nodes(n).nodalLoad(6);

            end
          end
        end

        %-----------------------------------------------------------------------
        % Assemble member fixed end force (FEF) vector in local system for a
        % uniformely distributed load applied to a given element.
        % Output:
        %  fel: fixed end force vector in local system
        % Input arguments:
        %  elem: handle to element object
        function fel = elemLocUniformLoadFEF(~,elem)
            include_constants;

            % Get element orientation angle cosine and sine
            cx = elem.cosine_X;
            cy = elem.cosine_Y;

            % Assemble fixed end force vector in local system.
            % Check whether load is given in global or local system and get axial
            % and flexural (transversal) end components of distributed load on member.
            unif = elem.uniformLoad;
            dir = elem.uniformDirection;

            if( dir == GLOBAL_LOAD )
%   *** COMPLETE HERE - Anm_Frame2D-03 ***




%   *** COMPLETE HERE - Anm_Frame2D-03 ***
            else
%   *** COMPLETE HERE - Anm_Frame2D-04 ***




%   *** COMPLETE HERE - Anm_Frame2D-04 ***
            end

            % Compute axial fixed end force components for a distributed
            % load applied on given member
            fea = elem.axialLinearLoadFEF(qxi,qxf);

            % Compute flexural (transversal) fixed end force components for a
            % distributed load applied on given member
            fef = elem.flexuralLinearLoadFEF_XY(qyi,qyf);

            % Assemble member fixed end force (FEF) vector in local system for a
            % frame beam element

%   *** COMPLETE HERE - Anm_Frame2D-05 ***






%   *** COMPLETE HERE - Anm_Frame2D-05 ***
        end
        
        %-----------------------------------------------------------------------
        % Assemble member fixed end force (FEF) vector in local system for a
        % linearly distributed load applied to a given element.
        % Output:
        %  fel: fixed end force vector in local system
        % Input arguments:
        %  elem: handle to element object
        function fel = elemLocLinearLoadFEF(~,elem)
            include_constants;

            % Get element orientation angle cosine and sine
            cx = elem.cosine_X;
            cy = elem.cosine_Y;

            % Assemble fixed end force vector in local system.
            % Check whether load is given in global or local system and get axial
            % and flexural (transversal) end components of distributed load on member.
            line = elem.linearLoad;
            dir = elem.linearDirection;

            if( dir == GLOBAL_LOAD )
%   *** COMPLETE HERE - Anm_Frame2D-06 ***




%   *** COMPLETE HERE - Anm_Frame2D-06 ***
            else
%   *** COMPLETE HERE - Anm_Frame2D-07 ***




%   *** COMPLETE HERE - Anm_Frame2D-07 ***
            end

            % Compute axial fixed end force components for a distributed
            % load applied on given member
            fea = elem.axialLinearLoadFEF(qxi,qxf);

            % Compute flexural (transversal) fixed end force components for a
            % distributed load applied on given member
            fef = elem.flexuralLinearLoadFEF_XY(qyi,qyf);

            % Assemble member fixed end force (FEF) vector in local system for a
            % frame beam element

%   *** COMPLETE HERE - Anm_Frame2D-08 ***






%   *** COMPLETE HERE - Anm_Frame2D-08 ***
        end

        %-----------------------------------------------------------------------
        % Assemble member fixed end force (FEF) vector in local system for a
        % thermal expansion of a given element.
        % Output:
        %  fel: fixed end force vector in local system
        % Input arguments:
        %  elem: handle to element object
        function fel = elemLocThermalLoadFEF(~,elem)
            include_constants;

            % Compute axial fixed end force components for a thermal expansion
            % of a given member
            fea = elem.axialThermalLoadFEF();

            % Compute flexural (transversal) fixed end force components for a
            % thermal expansion of a given member
            fef = elem.flexuralThermalLoadFEF_XY();

            % Assemble member fixed end force (FEF) vector in local system for a
            % truss element

%   *** COMPLETE HERE - Anm_Frame2D-09 ***






%   *** COMPLETE HERE - Anm_Frame2D-09 ***
        end

        %-----------------------------------------------------------------------
        % Initialize given element internal force arrays with null values.
        %  axial_force(1,2)
        %   Ni = axial_force(1) - init value
        %   Nf = axial_force(2) - final value
        %  shear_force(1,2)
        %   Qi = shear_force(1) - init value
        %   Qf = shear_force(2) - final value
        %  bending_moment(1,2)
        %   Mi = bending_moment(1) - init value
        %   Mf = bending_moment(2) - final value
        % Input arguments:
        %  elem: handle to element object
        function obj = initIntForce(obj,elem)
            elem.axial_force = zeros(1,2);
            elem.shear_force_Y = zeros(1,2);
            elem.bending_moment_Z = zeros(1,2);
        end

        %-----------------------------------------------------------------------
        % Assemble contribution of given internal force vector (fel) to
        % arrays of internal forces of given element (elem).
        % Input arguments:
        %  elem: handle to element object
        %  fel: element internal force vector in local system
        function obj = assembleIntForce(obj,elem,fel)
            elem.axial_force(1) = elem.axial_force(1) + fel(1);
            elem.axial_force(2) = elem.axial_force(2) + fel(4);
            elem.shear_force_Y(1) = elem.shear_force_Y(1) + fel(2);
            elem.shear_force_Y(2) = elem.shear_force_Y(2) + fel(5);
            elem.bending_moment_Z(1) = elem.bending_moment_Z(1) + fel(3);
            elem.bending_moment_Z(2) = elem.bending_moment_Z(2) + fel(6);
        end
    end
end