%% Anm_Grillage class
%
% This is a sub-class, in the Object Oriented Programming (OOP) paradigm,
% of super-class <anm.html *Anm*> in the <main.html LESM (Linear Elements 
% Structure Model)> program. This sub-class implements abstract methods, 
% defined in super-class *Anm*, that deal with grillage analysis model
% of linear structure elements.
%
%% Authors
% Luiz Fernando Martha and Rafael Lopez Rangel
%
%% Grillage analysis
% A grillage model is a common form of analysis model for building
% stories and bridge decks. Its key features are:
% It is a 2D model, which in LESM is considered on the XY plane.
  %%%
% * Beam members are laid out in a grid pattern in a single plane,
%   rigidly connected at nodes. However, a grillage member might have
%   a hinge (rotation liberation) at an end or hinges at both ends.
%   In LESM, it is assumed that a hinge in a grillage member releases
%   continuity of both bending and torsion rotations.
% * By assumption, there is only out-of-plane behavior, which
%   includes displacement transversal to the grillage plane, and
%   rotations about axes on the plane.
% * Internal forces at any cross-section of a grillage member are:
%   shear force (in the plane transversal direction), bending moment
%   (bending of members occurs in a plane perpendicular to the
%   grillage plane), and torsion moment (about a member axis).
%   By assumption, there is no axial force in a grillage member.
% * The geometric parameters of a grillage member cross-section are
%   its effective shear area, its moment of inertia for bending
%   behavior, and its moment of inertia for torsion behavior.
% * Each node of a grillage model has three d.o.f.'s: a transversal
%   displacement in Z direction, a rotation about the plane X axis,
%   and a rotation about the plane Y axis.
%
%% History
% @version 1.00
%
% Initial version: August-December 2016
%%%
% Prepared for the course ENG 1240 - Anlise de Estruturas III, 2016.2,
% Department of Civil Engineering, PUC-Rio.
%
classdef Anm_Grillage < Anm
    %%
    % <anm.html See documentation on *Anm* super-class>.

    %% Constructor method
    methods
        %-----------------------------------------------------------------------
        function obj = Anm_Grillage()
            include_constants;
            obj = obj@Anm(GRILLAGE_ANALYSIS,3);
        end
    end

    %% Public methods
    % Implementation of the abstract methods declared in super-class <anm.html *Anm*>.
    methods
        %-----------------------------------------------------------------------
        % Assemble rotation transformation matrix from global system to local system
        % Output:
        %  rot:   rotation transformation matrix
        % Input arguments:
        %  elem:  handle to element object
        function rot = gblToLocElemRotMtx(~,elem)
            % Get element orientation angle cosine and sine
            cx = elem.cosine_X;            % element orientation cosine
            cy = elem.cosine_Y;            % element orientation sine
            
            % Assemble rotation transformation matrix
%   *** COMPLETE HERE - Anm_Grillage-01 ***






%   *** COMPLETE HERE - Anm_Grillage-01 ***
        end

        %-----------------------------------------------------------------------
        % Setup global d.o.f. (degree of freedown) numbering ID matrix.
        % Free d.o.f.'s are numbered first.
        % Counts total number of equations of free d.o.f.'s and total number
        % of equations of fixed d.o.f.'s, and stores these numbers in global
        % variables.
        % Initialize ID matrix:
        %  if ID(k,n) = 0, d.o.f. k of node n is free
        %  if ID(k,n) = 1, d.o.f. k of node n is fixed
        function setupDOFNum(obj,drv)
            % Dimension global d.o.f. numbering matrix
            drv.ID = zeros(obj.ndof,drv.nnp);

            % Compute total number of fixed d.o.f.
            drv.neqfixed = 0;
            for n = 1:drv.nnp
                if drv.nodes(n).ebc(4) == 1
                    drv.neqfixed = drv.neqfixed + 1;
                    drv.ID(1,n) = 1;
                end
                if drv.nodes(n).ebc(5) == 1
                    drv.neqfixed = drv.neqfixed + 1;
                    drv.ID(2,n) = 1;
                end
                if drv.nodes(n).ebc(3) == 1
                    drv.neqfixed = drv.neqfixed + 1;
                    drv.ID(3,n) = 1;
                end
            end

            % Compute total number of free d.o.f.
            drv.neqfree = drv.neq - drv.neqfixed;
        end

        %-----------------------------------------------------------------------
        % Store prescribed displacements in global displacement vector
        % (known support settlement values).
        % Avoid storing a prescribed displacement component in a position of
        % the global displacement vector that corresponds to a free d.o.f.
        function setupPrescDispl(~,drv)
          for n = 1:drv.nnp
            if size(drv.nodes(n).prescDispl,2) > 0

              if (drv.ID(1,n) > drv.neqfree) && ...
                 (drv.nodes(n).prescDispl(4) ~= 0)
                drv.D(drv.ID(1,n)) = drv.nodes(n).prescDispl(4);
              end

              if (drv.ID(2,n) > drv.neqfree) && ...
                 (drv.nodes(n).prescDispl(5) ~= 0)
                drv.D(drv.ID(2,n)) = drv.nodes(n).prescDispl(5);
              end

              if (drv.ID(3,n) > drv.neqfree) && ...
                 (drv.nodes(n).prescDispl(3) ~= 0)
                drv.D(drv.ID(3,n)) = drv.nodes(n).prescDispl(3);
              end

            end
          end
        end

        %-----------------------------------------------------------------------
        % Assemble member stiffness matrix in local system for a given element
        % according to target analisys model.
        % Output:
        %  kel:   target element stiffness matrix in local system
        % Input arguments:
        %  elem:  handle to element object
        function kel = elemLocStiffMtx(~,elem)
            % Assemble element stiffness matrix in local system
            % Compute torsion stiffness coefficients for given member
            ket = elem.torsionStiffCoeff();

            % Compute flexural stiffness coefficients for given member
            % assuming no rotation liberation at nodes
            kef = elem.flexuralStiffCoeff_XZ();

%   *** COMPLETE HERE - Anm_Grillage-02 ***






%   *** COMPLETE HERE - Anm_Grillage-02 ***
        end

        %-----------------------------------------------------------------------
        % Add nodal load components to any term of the global forcing vector,
        % incluing the terms that correspond to constrained d.o.f.
        function nodalLoads(~,drv)
            for n = 1:drv.nnp
                if size(drv.nodes(n).nodalLoad,2) > 0

                    id = drv.ID(1,n);
                    drv.F(id) = drv.F(id) + drv.nodes(n).nodalLoad(4);

                    id = drv.ID(2,n);
                    drv.F(id) = drv.F(id) + drv.nodes(n).nodalLoad(5);

                    id = drv.ID(3,n);
                    drv.F(id) = drv.F(id) + drv.nodes(n).nodalLoad(3);

                end
            end
        end

        %-----------------------------------------------------------------------
        % Assemble member fixed end force (FEF) vector in local system for a
        % uniformely distributed load applied to a given element according to
        % target analisys model.
        % Output:
        %  fel:   fixed end force vector in local system
        % Input arguments:
        %  elem:  handle to element object
        function fel = elemLocUniformLoadFEF(~,elem)
            % Assemble fixed end force vector in local system
            % Get flexural (transversal) end components of distributed load on member
            % (always in z global axis)
            qzi = elem.uniformLoad(3);
            qzf = elem.uniformLoad(3);

            % Compute flexural fixed end  force components for a distributed load
            % applied on given member, assuming no rotation liberation at nodes
            fef = elem.flexuralLinearLoadFEF_XZ(qzi,qzf);

            % Assemble member fixed end force (FEF) vector in local system for a
            % grillage beam element

%   *** COMPLETE HERE - Anm_Grillage-03 ***






%   *** COMPLETE HERE - Anm_Grillage-03 ***
        end

        %-----------------------------------------------------------------------
        % Assemble member fixed end force (FEF) vector in local system for a
        % linearly distributed load applied to a given element according to
        % target analisys model.
        % Output:
        %  fel:  fixed end force vector in local system
        % Input arguments:
        %  elem: handle to element object
        function fel = elemLocLinearLoadFEF(~,elem)
            % Assemble fixed end force vector in local system
            % Get flexural (transversal) end components of distributed load on member
            % (always in z global axis)
            qzi = elem.linearLoad(3);
            qzf = elem.linearLoad(6);

            % Compute flexural fixed end  force components for a distributed load
            % applied on given member, assuming no rotation liberation at nodes
            fef = elem.flexuralLinearLoadFEF_XZ(qzi,qzf);

            % Assemble member fixed end force (FEF) vector in local system for a
            % grillage beam element

%   *** COMPLETE HERE - Anm_Grillage-04 ***






%   *** COMPLETE HERE - Anm_Grillage-04 ***
        end

        %-----------------------------------------------------------------------
        % Assemble member fixed end force (FEF) vector in local system for a
        % thermal expansion of a given element according to target analisys model.
        % Output:
        %  fel:   fixed end force vector in local system
        % Input arguments:
        %  elem:  handle to element object
        function fel = elemLocThermalLoadFEF(~,elem)
            include_constants;

            % Compute flexural (transversal) fixed end force components for a
            % thermal expansion of a given member
            fef = elem.flexuralThermalLoadFEF_XZ();

            % Assemble member fixed end force (FEF) vector in local system for a
            % truss element

%   *** COMPLETE HERE - Anm_Grillage-05 ***






%   *** COMPLETE HERE - Anm_Grillage-05 ***
        end

        %-----------------------------------------------------------------------
        % Initialize given element internal force arrays with null values.
        %  torsion_moment(1,2)
        %   Ti = torsion_moment(1) - init value
        %   Tf = torsion_moment(2) - final value
        %  bending_moment(1,2)
        %   Mi = bending_moment(1) - init value
        %   Mf = bending_moment(2) - final value
        %  shear_force(1,2)
        %   Qi = shear_force(1) - init value
        %   Qf = shear_force(2) - final value
        % Input arguments:
        %  elem:   handle to element object
        function obj = initIntForce(obj,elem)
            elem.torsion_moment = zeros(1,2);
            elem.bending_moment_Y = zeros(1,2);
            elem.shear_force_Z = zeros(1,2);
        end

        %-----------------------------------------------------------------------
        % Assemble contribution of given internal force vector (fel) to
        % arrays of internal forces of given element (elem).
        % Input arguments:
        %  elem:   handle to element object
        %  fel: element internal force vector in local system
        function obj = assembleIntForce(obj,elem,fel)
            elem.torsion_moment(1) = elem.torsion_moment(1) + fel(1);
            elem.torsion_moment(2) = elem.torsion_moment(2) + fel(4);
            elem.bending_moment_Y(1) = elem.bending_moment_Y(1) + fel(2);
            elem.bending_moment_Y(2) = elem.bending_moment_Y(2) + fel(5);
            elem.shear_force_Z(1) = elem.shear_force_Z(1) + fel(3);
            elem.shear_force_Z(2) = elem.shear_force_Z(2) + fel(6);
        end
    end
end