%% Anm_Truss2D class
%
% This is a sub-class, in the Object Oriented Programming (OOP) paradigm,
% of super-class <anm.html *Anm*> in the <main.html LESM (Linear Elements 
% Structure Model)> program. This sub-class implements abstract methods, 
% defined in super-class *Anm*, that deal with planar truss analysis model
% of linear structure elements.
%
%% Authors
% Luiz Fernando Martha and Rafael Lopez Rangel
%
%% Truss 2D analysis
% A truss model is a common form of analysis model, with the
% following assumptions:
  %%%
% * Truss members are bars connected at their ends only, and they are
%   connected by friction-less pins. Therefore, a truss member does
%   not present any secondary bending moment induced by rotation
%   continuity at joints.
% * A truss model is loaded only at joints, which are also called
%   nodes. Any load action along a member, such as self weight, is
%   statically transferred as concentrated forces to the member end
%   nodes.
% * Local bending of members due to transversal member loads are
%   neglected, when compared to the effect of global load acting on
%   the truss.
% * Therefore, there is only one type of internal force in a truss
%   member: axial internal force, which may be tension or
%   compression.
% * As truss members have (by assumption) only axial behavior, the
%   area of its cross-section is the single geometric parameter
%   needed.
% * A 2D truss model is laid on the XY plane.
% * Each node of a 2D truss model has two d.o.f.'s (degrees of
%   freedom): a horizontal displacement in X direction and a vertical
%   displacement in Y direction.
%
%% History
% @version 1.00
%
% Initial version: August-December 2016
%%%
% Prepared for the course ENG 1240 - Anlise de Estruturas III, 2016.2,
% Department of Civil Engineering, PUC-Rio.
%
classdef Anm_Truss2D < Anm
    %%
    % <anm.html See documentation on *Anm* super-class>.

    %% Constructor method
    methods
        %-----------------------------------------------------------------------
        function obj = Anm_Truss2D()
            include_constants;
            obj = obj@Anm(TRUSS2D_ANALYSIS,2);
        end
    end

    %% Public methods
    % Implementation of the abstract methods declared in super-class <anm.html *Anm*>.
    methods
        %-----------------------------------------------------------------------
        % Assemble rotation transformation matrix from global system to local system
        % Output:
        %  rot:   rotation transformation matrix
        % Input arguments:
        %  elem:  handle to element object
        function rot = gblToLocElemRotMtx(~,elem)
            % Get element orientation angle cosine and sine
            cx = elem.cosine_X;            % element orientation cosine
            cy = elem.cosine_Y;            % element orientation sine

            % Assemble rotation transformation matrix
%   *** COMPLETE HERE - Anm_Truss2D-01 ***




%   *** COMPLETE HERE - Anm_Truss2D-01 ***
        end

        %-----------------------------------------------------------------------
        % Setup global d.o.f. (degree of freedown) numbering ID matrix.
        % Free d.o.f.'s are numbered first.
        % Counts total number of equations of free d.o.f.'s and total number
        % of equations of fixed d.o.f.'s, and stores these numbers in global
        % variables.
        % Initialize ID matrix:
        %  if ID(k,n) = 0, d.o.f. k of node n is free
        %  if ID(k,n) = 1, d.o.f. k of node n is fixed
        function setupDOFNum(obj,drv)
            % Dimension global d.o.f. numbering matrix
            drv.ID = zeros(obj.ndof,drv.nnp);

            % Compute total number of fixed d.o.f.
            drv.neqfixed = 0;
            for n = 1:drv.nnp
                if drv.nodes(n).ebc(1) == 1
                    drv.neqfixed = drv.neqfixed + 1;
                    drv.ID(1,n) = 1;
                end
                if drv.nodes(n).ebc(2) == 1
                    drv.neqfixed = drv.neqfixed + 1;
                    drv.ID(2,n) = 1;
                end
            end
            
            % Compute total number of free d.o.f.
            drv.neqfree = drv.neq - drv.neqfixed;
        end

        %-----------------------------------------------------------------------
        % Store prescribed displacements in global displacement vector
        % (known support settlement values).
        % Avoid storing a prescribed displacement component in a position of
        % the global displacement vector that corresponds to a free d.o.f.
        function setupPrescDispl(~,drv)
          for n = 1:drv.nnp
            if size(drv.nodes(n).prescDispl,2) > 0

              if (drv.ID(1,n) > drv.neqfree) && ...
                 (drv.nodes(n).prescDispl(1) ~= 0)
                drv.D(drv.ID(1,n)) = drv.nodes(n).prescDispl(1);
              end

              if (drv.ID(2,n) > drv.neqfree) && ...
                 (drv.nodes(n).prescDispl(2) ~= 0)
                drv.D(drv.ID(2,n)) = drv.nodes(n).prescDispl(2);
              end

            end
          end
        end

        %-----------------------------------------------------------------------
        % Assemble member stiffness matrix in local system for a given element
        % according to target analisys model.
        % Output:
        %  kel:   target element stiffness matrix in local system
        % Input arguments:
        %  elem:  handle to element object
        function kel = elemLocStiffMtx(~,elem)
            % Assemble element stiffness matrix in local system
            % Compute axial stiffness coefficients for given member
            kea = elem.axialStiffCoeff();

%   *** COMPLETE HERE - Anm_Truss2D-02 ***




%   *** COMPLETE HERE - Anm_Truss2D-02 ***
        end

        %-----------------------------------------------------------------------
        % Add nodal load components to any term of the global forcing vector,
        % incluing the terms that correspond to constrained d.o.f.
        function nodalLoads(~,drv)
            for n = 1:drv.nnp
                if size(drv.nodes(n).nodalLoad,2) > 0

                    id = drv.ID(1,n);
                    drv.F(id) = drv.F(id) + drv.nodes(n).nodalLoad(1);

                    id = drv.ID(2,n);
                    drv.F(id) = drv.F(id) + drv.nodes(n).nodalLoad(2);

                end
            end
        end

        %-----------------------------------------------------------------------
        % Assemble member fixed end force (FEF) vector in local system for a
        % uniformely distributed load applied to a given element according to
        % target analisys model.
        % Output:
        %  fel:   fixed end force vector in local system
        % Input arguments:
        %  elem:  handle to element object
        function fel = elemLocUniformLoadFEF(~,elem)
            include_constants;

            % Get element orientation angle cosine and sine
            cx = elem.cosine_X;            % element orientation cosine
            cy = elem.cosine_Y;            % element orientation sine

            % Assemble fixed end force vector in local system
            % Check whether load is given in global or local system and get axial
            % and transversal end components of distributed load on member
            unif = elem.uniformLoad;
            dir = elem.uniformDirection;

            if( dir == GLOBAL_LOAD )
%   *** COMPLETE HERE - Anm_Truss2D-03 ***




%   *** COMPLETE HERE - Anm_Truss2D-03 ***
            else
%   *** COMPLETE HERE - Anm_Truss2D-04 ***




%   *** COMPLETE HERE - Anm_Truss2D-04 ***
            end

            % Compute axial fixed end force components for a distributed
            % load applied on given member
            fea = elem.axialLinearLoadFEF(qxi,qxf);

            % Compute flexural (transversal) fixed end force components for a
            % distributed load applied on given member, assuming doubly hinged ends
            fes = elem.simplySuppLinearLoadFEF(qyi,qyf);

            % Assemble member fixed end force (FEF) vector in local system for a
            % truss element

%   *** COMPLETE HERE - Anm_Truss2D-05 ***




%   *** COMPLETE HERE - Anm_Truss2D-05 ***
        end

        %-----------------------------------------------------------------------
        % Assemble member fixed end force (FEF) vector in local system for a
        % linearly distributed load applied to a given element according to
        % target analisys model.
        % Output:
        %  fel:  fixed end force vector in local system
        % Input arguments:
        %  elem: handle to element object
        function fel = elemLocLinearLoadFEF(~,elem)
            include_constants;

            % Get element orientation angle cosine and sine
            cx = elem.cosine_X;            % element orientation cosine
            cy = elem.cosine_Y;            % element orientation sine

            % Assemble fixed end force vector in local system
            % Check whether load is given in global or local system and get axial
            % and transversal end components of distributed load on member
            line = elem.linearLoad;
            dir = elem.linearDirection;

            if( dir == GLOBAL_LOAD )
%   *** COMPLETE HERE - Anm_Truss2D-06 ***




%   *** COMPLETE HERE - Anm_Truss2D-06 ***
            else
%   *** COMPLETE HERE - Anm_Truss2D-07 ***




%   *** COMPLETE HERE - Anm_Truss2D-07 ***
            end

            % Compute axial fixed end force components for a distributed
            % load applied on given member
            fea = elem.axialLinearLoadFEF(qxi,qxf);

            % Compute flexural (transversal) fixed end force components for a
            % distributed load applied on given member, assuming doubly hinged ends
            fes = elem.simplySuppLinearLoadFEF(qyi,qyf);

            % Assemble member fixed end force (FEF) vector in local system for a
            % truss element

%   *** COMPLETE HERE - Anm_Truss2D-08 ***




%   *** COMPLETE HERE - Anm_Truss2D-08 ***
        end

        %-----------------------------------------------------------------------
        % Assemble member fixed end force (FEF) vector in local system for a
        % thermal expansion of a given element according to target analisys model.
        % Output:
        %  fel:   fixed end force vector in local system
        % Input arguments:
        %  elem:  handle to element object
        function fel = elemLocThermalLoadFEF(~,elem)
            % Compute axial fixed end force components for a thermal expansion
            % of a given member
            fea = elem.axialThermalLoadFEF();

            % Assemble member fixed end force (FEF) vector in local system for a
            % truss element

%   *** COMPLETE HERE - Anm_Truss2D-09 ***




%   *** COMPLETE HERE - Anm_Truss2D-09 ***
        end

        %-----------------------------------------------------------------------
        % Initialize given element internal force arrays with null values.
        % Initialize array of axial forces at member ends
        %  In TRUSS2D_ANALYSIS:
        %  axial_force(1,2)
        %   Ni = axial_force(1) - init value
        %   Nf = axial_force(2) - final value
        % Input arguments:
        %  elem:   handle to element object
        function obj = initIntForce(obj,elem)
            elem.axial_force = zeros(1,2);
        end

        %-----------------------------------------------------------------------
        % Assemble contribution of given internal force vector (fel) to
        % arrays of internal forces of given element (elem).
        % Input arguments:
        %  elem:   handle to element object
        %  fel: element internal force vector in local system
        function obj = assembleIntForce(obj,elem,fel)
            elem.axial_force(1) = elem.axial_force(1) + fel(1);
            elem.axial_force(2) = elem.axial_force(2) + fel(3);
        end
    end
end