%% Drv class
%
% This is a class in the Object Oriented Programming (OOP) paradigm
% that defines an object that drives the structural analysis in the
% <main.html LESM (Linear Elements Structure Model)> program.
%
% The main global variables of a typical structural analysis program
% are attributes of a Drv object.
%
%% Authors
% Luiz Fernando Martha and Rafael Lopez Rangel
%
%% History
% @version 1.00
%
% Initial version: August-December 2016
%%%
% Prepared for the course ENG 1240 - Anlise de Estruturas III, 2016.2,
% Department of Civil Engineering, PUC-Rio.
%
classdef Drv < hgsetget
    %%
    % <http://www.mathworks.com/help/matlab/handle-classes.html
    % See documentation on *hgsetget* handle super-class>.

    %% Public attributes
    properties (SetAccess = public, GetAccess = public)
        anm = [];       % handle to analysis model object
        nodes = [];     % vector of node object handles
        elems = [];     % vector of element object handles
        K = [];         % global stiffness matrix
        F = [];         % global system forcing vector
        D = [];         % global system displacement vector
        ID = [];        % global d.o.f. numbering matrix
        nnp = 0;        % number of nodal points
        nel = 0;        % number of elements
        neq = 0;        % number of equations
        neqfree = 0;    % number of equations of free d.o.f.
        neqfixed = 0;   % number of equations of fixed d.o.f.
    end

    %% Constructor method
    methods
        %-----------------------------------------------------------------------
        function obj = Drv(anm,nodes,elems,K,F,D,ID,nnp,nel,neq,neqfr,neqfx)
            if (nargin > 0)
                obj.anm = anm;          % Store given handle to analysis model object
                obj.nodes = nodes;      % Store given vector of node object handles
                obj.elems = elems;      % Store given vector of element object handles
                obj.K = K;              % Store given global stiffness matrix
                obj.F = F;              % Store given global system forcing vector
                obj.D = D;              % Store given global system displacement vector
                obj.ID = ID;            % Store given global d.o.f. numbering matrix
                obj.nnp = nnp;          % Store given number of nodal points
                obj.nel = nel;          % Store given number of elements
                obj.neq = neq;          % Store given number of equations
                obj.neqfree = neqfr;    % Store given number of equations of free d.o.f.
                obj.neqfixed = neqfx;   % Store given number of equations of fixed d.o.f.
            end
        end
    end

    %% Public set methods
    methods
        %-----------------------------------------------------------------------
        % Set analysis model parameter value
        function obj = set.anm(obj,anm)
            obj.anm = anm;
        end

        %-----------------------------------------------------------------------
        % Set vector of node object handles parameter value
        function obj = set.nodes(obj,nodes)
            obj.nodes = nodes;
        end

        %-----------------------------------------------------------------------
        % Set vector of element object handles parameter value
        function obj = set.elems(obj,elems)
            obj.elems = elems;
        end

        %-----------------------------------------------------------------------
        % Set global stiffness matrix parameter value
        function obj = set.K(obj,K)
            obj.K = K;
        end

        %-----------------------------------------------------------------------
        % Set global system forcing vector parameter value
        function obj = set.F(obj,F)
            obj.F = F;
        end

        %-----------------------------------------------------------------------
        % Set global system displacement vector parameter value
        function obj = set.D(obj,D)
            obj.D = D;
        end

        %-----------------------------------------------------------------------
        % Set global d.o.f. numbering matrix parameter value
        function obj = set.ID(obj,ID)
            obj.ID = ID;
        end

        %-----------------------------------------------------------------------
        % Set number of nodal points and number of equations parameter value
        function obj = set.nnp(obj,nnp)
            obj.nnp = nnp;
        end

        %-----------------------------------------------------------------------
        % Set number of elements parameter value
        function obj = set.nel(obj,nel)
            obj.nel = nel;
        end

        %-----------------------------------------------------------------------
        % Set number of d.o.f. equations parameter value
        function obj = set.neq(obj,neq)
            obj.neq = neq;
        end

        %-----------------------------------------------------------------------
        % Set number of equations of free d.o.f. parameter value
        function obj = set.neqfree(obj,neqfr)
            obj.neqfree = neqfr;
        end

        %-----------------------------------------------------------------------
        % Set number of equations of fixed d.o.f. parameter value
        function obj = set.neqfixed(obj,neqfx)
            obj.neqfixed = neqfx;
        end
    end

    %% Public get methods
    methods
        %-----------------------------------------------------------------------
        % Get analysis model parameter value
        function anm = get.anm(obj)
            anm = obj.anm;
        end

        %-----------------------------------------------------------------------
        % Get vector of node object handles parameter value
        function nodes = get.nodes(obj)
            nodes = obj.nodes;
        end

        %-----------------------------------------------------------------------
        % Get vector of element object handles parameter value
        function elems = get.elems(obj)
            elems = obj.elems;
        end

        %-----------------------------------------------------------------------
        % Get global stiffness matrix parameter value
        function K = get.K(obj)
            K = obj.K;
        end

        %-----------------------------------------------------------------------
        % Get global system forcing vector parameter value
        function F = get.F(obj)
            F = obj.F;
        end

        %-----------------------------------------------------------------------
        % Get global system displacement vector parameter value
        function D = get.D(obj)
            D = obj.D;
        end

        %-----------------------------------------------------------------------
        % Get global d.o.f. numbering matrix parameter value
        function ID = get.ID(obj)
            ID = obj.ID;
        end

        %-----------------------------------------------------------------------
        % Get number of nodal points parameter value
        function nnp = get.nnp(obj)
            nnp = obj.nnp;
        end

        %-----------------------------------------------------------------------
        % Get number of elements parameter value
        function nel = get.nel(obj)
            nel = obj.nel;
        end

        %-----------------------------------------------------------------------
        % Get number of equations parameter value
        function neq = get.neq(obj)
            neq = obj.neq;
        end

        %-----------------------------------------------------------------------
        % Get number of equations of free d.o.f. parameter value
        function neqfr = get.neqfree(obj)
            neqfr = obj.neqfree;
        end

        %-----------------------------------------------------------------------
        % Get number of equations of fixed d.o.f parameter value
        function neqfx = get.neqfixed(obj)
            neqfx = obj.neqfixed;
        end
    end

    %% Class (static) auxiliary functions
    methods (Static)
        %-----------------------------------------------------------------------
        % Open neutral-format file of a structure model.
        % Output:
        %  nf: integer identifier of input file
        function nf = openNF
            filterspec = {'*.nf';'*.dat';'*.pos'};
            [filename,pathname] = uigetfile(filterspec,'LESM - Input file');
            if isequal(filename,0)
                nf = 0;
            else
                fullname = strcat(pathname,filename);
                nf = fopen(fullname,'rt');
            end
        end
    end

    %% Public methods
    methods
        %-----------------------------------------------------------------------
        % Dimension and initialize global matrix, forcing vector and system displacement vector.
        function obj = dimKFD(obj)
            obj.K = zeros(obj.neq,obj.neq);   % initialize global stiffness matrix
            obj.F = zeros(obj.neq,1);         % initialize global system forcing vector
            obj.D = zeros(obj.neq,1);         % initialize global system displacement vector
        end

        %-----------------------------------------------------------------------
        % Assemble ID matrix:
        %  ID(k,n) = equation number of d.o.f. k of node n
        %  Free d.o.f.'s have the initial numbering
        %  countf --> counts free or natural B.C. (unknown) d.o.f. (numbered first)
        %  countc --> counts constrainted B.C. (essential - known) d.o.f.
        function obj = assembleDOFNum(obj)
            countf = 0;
            countc = obj.neqfree;

            for n = 1:obj.nnp
                for k = 1:obj.anm.ndof

                    if(obj.ID(k,n) == 0)
                        countf = countf + 1;
                        obj.ID(k,n) = countf;
                    else
                        countc = countc + 1;
                        obj.ID(k,n) = countc;
                    end

                end
            end
        end

        %-----------------------------------------------------------------------
        % Assemble element gather vector (stores element d.o.f. eqn. numbers).
        function obj = assembleGle(obj)
            for e = 1:obj.nel
                obj.elems(e).gle = zeros(obj.elems(e).nen*obj.anm.ndof,1);
                i = 0;

                for j = 1:obj.anm.ndof
                    i = i + 1;
                    obj.elems(e).gle(i) = obj.ID(j,obj.elems(e).nodei.id);
                end

                for j = 1:obj.anm.ndof
                    i = i + 1;
                    obj.elems(e).gle(i) = obj.ID(j,obj.elems(e).nodef.id);
                end

            end
        end

        %-----------------------------------------------------------------------
        % Assemble global stiffness matrix (add the contribution of all elements
        % to the global stiffness matrix).
        function obj = gblMtx(obj)
            for e = 1:obj.nel
                keg = obj.elems(e).gblStiffMtx();
%   *** COMPLETE HERE - Drv-01 **

%   *** COMPLETE HERE - Drv-01 **
            end
        end

        %-----------------------------------------------------------------------
        % Assemble given element stiffness matrix (in global system) to
        % global stiffness matrix.
        % Input arguments:
        %  keg: element local stiffness matrix in global system
        %  elem: handle to element object
        function obj = assembleElemMtx(obj,keg,elem)
            gle = elem.gle;

            % Add contribution of element given by its id to global stiffness matrix
            % il is the local (element) row equation number
            % jl is the local (element) column equation number
            % ig is the global row equation number
            % jg is the global column equation number
            ndof = obj.anm.get('ndof');
            nen = elem.get('nen');
%   *** COMPLETE HERE - Drv-02 **







%   *** COMPLETE HERE - Drv-02 **
        end

        %-----------------------------------------------------------------------
        % Add element equivalent nodal loads to global forcing vector.
        function obj = elemLoads(obj)

            % Add uniformely distributed element loads as equivalent
            % nodal loads (ENL) to forcing vector
            for e = 1:obj.nel
                if size(obj.elems(e).uniformLoad,2) > 0
                    feg = obj.elems(e).gblUniformLoadENL();
%   *** COMPLETE HERE - Drv-03 **

%   *** COMPLETE HERE - Drv-03 **
                end
            end

            % Add linearly distributed element loads as equivalent
            % nodal loads (ENL) to forcing vector
            for e = 1:obj.nel
                if size(obj.elems(e).linearLoad,2) > 0
                    feg = obj.elems(e).gblLinearLoadENL();
%   *** COMPLETE HERE - Drv-04 **

%   *** COMPLETE HERE - Drv-04 **
                end
            end
        end

        %-----------------------------------------------------------------------
        % Add equivalent nodal load vector (in global system) to the global
        % forcing vector.
        % Assemble the given equivalent nodal load vector to any term of
        % the global forcing vector, incluing the terms that correspond
        % to constrained d.o.f.
        % Input arguments:
        %  elem: handle to element object
        %  feg: equivalent nodal load vector in global system
        function obj = assembleENL(obj,elem,feg)
            gle = elem.gle;

            % Add contribution of equivalent nodal load vector to global forcing vector
            % il is the local (element) row equation number
            % ig is the global row equation number
            ndof = anm.get('ndof');
            nen = elem.get('nen');
%   *** COMPLETE HERE - Drv-05 **




%   *** COMPLETE HERE - Drv-05 **
        end

        %-----------------------------------------------------------------------
        % Partition and solve the system of equations.
        % Partition the coefficient matrix K, forcing vector F and unknown vector D:
        %  f --> free or natural B.C. (unknown) degree-of-freedown (numbered first)
        %  c --> constrainted (essential - known) B.C. degree-of-freedown
        % Output:
        %  status: flag for stable free-free global matrix (0 -> unstable)
        function status = solveEqnSystem(obj)
            Kff = obj.K(1:obj.neqfree,1:obj.neqfree);                   % Extract Kff matrix
            % Check for stable free-free global matrix
            status = 1;
            if (rcond(Kff) < 10e-12)
                status = 0;
                return;
            end
            Kfc = obj.K(1:obj.neqfree,obj.neqfree+1:obj.neq);           % Extract Kfc matrix
            Kcf = obj.K(obj.neqfree+1:obj.neq,1:obj.neqfree);           % Extract Kcf matrix
            Kcc = obj.K(obj.neqfree+1:obj.neq,obj.neqfree+1:obj.neq);   % Extract Kcc matrix
            Ff  = obj.F(1:obj.neqfree);                                 % Extract Ff vector
            Dc  = obj.D(obj.neqfree+1:obj.neq);                         % Extract Dc vector
            Fc  = obj.F(obj.neqfree+1:obj.neq);                         % Extract Fc vector

            % Solve for Df
%   *** COMPLETE HERE - Drv-06 **

%   *** COMPLETE HERE - Drv-06 **

            % Reconstruct the global unknown vector D
            obj.D = [Df
                     Dc];

            % Recover forcing unknown values (reactions) at essential B.C.
            % It is assumed that the Fc vector currently stores combined nodal
            % loads applyed directly to fixed d.o.f's.
            % Superimpose computed reaction values to combined nodal loads,
            % with inversed direction, that were applyed directly to fixed d.o.f.'s.
            Fc = -Fc + Kcf*Df + Kcc*Dc;
            
            % Reconstruct the global forcing vector
            obj.F = [Ff
                     Fc];
        end

        %-----------------------------------------------------------------------
        % Compute element internal forces of each element depending on the
        % current analysis model:
        %  Add contribution of element internal forces from global analysis
        %  (from nodal displacements and rotations);
        %  Add contribution of element fixed end forces from
        %  uniformely distributed element loads;
        %  Add contribution of element fixed end forces from
        %  linearly distributed element loads;
        %  Add contribution of element fixed end forces from
        %  temperature variation;
        function obj = elemIntForce(obj)
            for e = 1:obj.nel
                % Initialize element internal force arrays with null values.
                obj.anm.initIntForce(obj.elems(e));

                % Add contribution of element internal forces from global analysis
                % (from nodal displacements and rotations)
%   *** COMPLETE HERE - Drv-07 **


%   *** COMPLETE HERE - Drv-07 **

                % Add contribution of element fixed end forces from
                % uniformely distributed element loads
                if size(obj.elems(e).uniformLoad,2) > 0
%   *** COMPLETE HERE - Drv-08 **


%   *** COMPLETE HERE - Drv-08 **
                end

                % Add contribution of element fixed end forces from
                % linearly distributed element loads
                if size(obj.elems(e).linearLoad,2) > 0
%   *** COMPLETE HERE - Drv-09 **


%   *** COMPLETE HERE - Drv-09 **
                end

                % Add contribution of element fixed end forces from
                % temperature variation
                if (obj.elems(e).tempVar_GC ~= 0) || ...
                   (obj.elems(e).tempVar_Y ~= 0)  || ...
                   (obj.elems(e).tempVar_Z ~= 0)
%   *** COMPLETE HERE - Drv-10 **


%   *** COMPLETE HERE - Drv-10 **
                end
            end
        end

        %-----------------------------------------------------------------------
        % Analyze current model data according to the direct stiffness
        % method, i.e., assemble global equilibrium system of equations,
        % calculate nodal displacements and rotations, and compute
        % reactions and elements internal forces.
        % Output:
        %  status: flag for stable free-free global matrix (0 -> unstable)
        function status = analyze(obj)
            fprintf(1,'Preparing analysis data...\n');
            
            % Dimension and initialize global matrix and vectors
            obj.dimKFD();

            % Generate global d.o.f. numbering matrix
            obj.anm.setupDOFNum(obj);

            % Assemble global d.o.f. numbering matrix
            obj.assembleDOFNum();

            % Assemble element gather vector (stores element d.o.f. eqn. numbers)
            obj.assembleGle();

            % Store prescribed displacements in global displacement vector
            obj.anm.setupPrescDispl(obj);

            % Assemble global stiffness matrix
            fprintf(1,'Assembling stiffness matrix...\n');
            obj.gblMtx();

            % Assemble global forcing vector
            fprintf(1,'Assembling forcing vector...\n');
            % Initialize forcing vector with nodal loads
            obj.anm.nodalLoads(obj);
            % Add distributed element loads as equivalent nodal loads to forcing vector
            obj.elemLoads();

            % Partition and solve system of equations
            fprintf(1,'Solving system of equations...\n');
            status = obj.solveEqnSystem();

            if (status == 1)
                % Add element internal forces to global arrays of internal forces
                fprintf(1,'Computing member internal forces...\n');
                obj.elemIntForce();
            else
                fprintf(1,'Unstable structure or invalid input data...\n');
            end
        end

        %-----------------------------------------------------------------------
        % Clean data structure of all objects.
        function obj = clean(obj)
            % Clean all Drv object properties
            obj.anm = [];
            obj.nodes = [];
            obj.elems = [];
            obj.K = [];
            obj.F = [];
            obj.D = [];
            obj.ID = [];
            obj.nnp = 0;
            obj.nel = 0;
            obj.neq = 0;
            obj.neqfree = 0;
            obj.neqfixed = 0;
        end
    end
end