%% Elem class
%
% This is an abstract super-class in the Object Oriented Programming (OOP)
% paradigm that generically defines a linear structure element object
% in the <main.html LESM (Linear Elements Structure Model)> program.
%
% This super-class is abstract because one cannot instantiate objects from
% it, since it contains abstract methods that should be implemented in
% (derived) sub-classes.
%
% This super-class defines object attributes and implement methods that
% define the generic behavior of linear structure elements. This generic
% behavior considers everything that does not depend on flexural behavior,
% including axial and torsional behavior.
%
% Therefore, derived sub-classes should implement abstract methods defined
% in this super-class that deals with flexural behavior.
%
% In addition, this super-class handles generically a three-dimensional
% behavior of a linear structure element in its local axes system.
% An object of class (Anm - Analysis model) is responsible to "project"
% this general 3D behavior to a specific model behavior, such as a planar
% frame, planar truss, or a grillage model. One of the attributes of an
% object of Elem class is a handle to a target Anm object.
%
%% Sub-classes
% In the current implementation of LESM program there are two sub-classes
% of this super-class:
  %%%
% * <elem_navier.html Navier (Euler-Bernoulli) flexural behavior>.
% * <elem_timoshenko.html Timoshenko flexural behavior>.
%
%% Authors
% Luiz Fernando Martha and Rafael Lopez Rangel
%
%% History
% @version 1.00
%
% Initial version: August-December 2016
%%%
% Prepared for the course ENG 1240 - Anlise de Estruturas III, 2016.2,
% Department of Civil Engineering, PUC-Rio.
%
classdef Elem < hgsetget
    %%
    % <http://www.mathworks.com/help/matlab/handle-classes.html
    % See documentation on *hgsetget* handle super-class>.

    %% Public attributes
    properties (SetAccess = public, GetAccess = public)
        nen = 2;                 % number of element nodes (always equal to 2)
        anm = [];                % handle to analysis model object
        type = 0;                % element type
        material = [];           % handle to material object
        section = [];            % handle to element cross section object
        nodei = [];              % handle to initial node object
        nodef = [];              % handle to final node object
        hingei = 0;              % flag for hinge at initial node
        hingef = 0;              % flag for hinge at final node
        length = 0;              % element length
        cosine_X = 0;            % element orientation cosine with X axis
        cosine_Y = 0;            % element orientation cosine with Y axis
        cosine_Z = 0;            % element orientation cosine with Z axis
        vz_X = 0;                % X component of auxiliary vector in local XZ plane 
        vz_Y = 0;                % Y component of auxiliary vector in local XZ plane 
        vz_Z = 0;                % Z component of auxiliary vector in local XZ plane 
        rot = [];                % rotation transformation matrix from global system to local system
        gle = [];                % element gather vector (stores element d.o.f. eqn. numbers)
        uniformLoad = [];        % vector of uniformly distributed load components
        uniformDirection = 0;    % flag for uniform load direction (local or global axes)
        linearLoad = []          % vector of linearly distributed load components
        linearDirection = 0;     % flag for linear load direction (local or global axes)
        tempVar_GC = 0;          % element temperature variation on its center of gravity line
        tempVar_Y = 0;           % element temperature variation relative to local Y axis
        tempVar_Z = 0;           % element temperature variation relative to local Z axis
        axial_force = [];        % array of axial internal forces at member ends
        shear_force_Y = [];      % array of shear internal forces at member ends
        shear_force_Z = [];      % array of shear internal forces at member ends
        bending_moment_Y = [];   % array of bending internal moments at member ends
        bending_moment_Z = [];   % array of bending internal moments at member ends
        torsion_moment = [];     % array of torsion internal moments at member ends
    end

    %% Constructor method
    methods
        %-----------------------------------------------------------------------
        function obj = Elem(anm,type,mat,sec,ni,nf,hi,hf,L,...
                            cx,cy,cz,vzx,vzy,vzz,rot,gle,unif,dirunif,...
                            lin,dirlin,tvgc,tvy,tvz,af,sfy,sfz,bmy,bmz,tm)
            if (nargin > 0)
                obj.anm = anm;
                obj.type = type;
                obj.material = mat;
                obj.section = sec;
                obj.nodei = ni;
                obj.nodef = nf;
                obj.hingei = hi;
                obj.hingef = hf;
                obj.length = L;
                obj.cosine_X = cx;
                obj.cosine_Y = cy;
                obj.cosine_Z = cz;
                obj.vz_X = vzx;
                obj.vz_Y = vzy;
                obj.vz_Z = vzz;
                obj.rot = rot;
                obj.gle = gle;
                obj.uniformLoad = unif;
                obj.uniformDirection = dirunif;
                obj.linearLoad = lin;
                obj.linearDirection = dirlin;
                obj.tempVar_GC = tvgc;
                obj.shear_force_Y = tvy;
                obj.shear_force_Z = tvz;
                obj.axial_force = af;
                obj.shear_force_Y = sfy;
                obj.shear_force_Z = sfz;
                obj.bending_moment_Y = bmy;
                obj.bending_moment_Z = bmz;
                obj.torsion_moment = tm;
            end
        end
    end

    %% Public set methods
    methods
        %-----------------------------------------------------------------------
        % Set analysis model handle
        function obj = set.anm(obj,anm)
            obj.anm = anm;
        end

        %-----------------------------------------------------------------------
        % Set element type parameter value
        function obj = set.type(obj,type)
            obj.type = type;
        end

        %-----------------------------------------------------------------------
        % Set material handle
        function obj = set.material(obj,mat)
            obj.material = mat;
        end

        %-----------------------------------------------------------------------
        % Set cross section handle
        function obj = set.section(obj,sec)
            obj.section = sec;
        end

        %-----------------------------------------------------------------------
        % Set initial node handle
        function obj = set.nodei(obj,ni)
            obj.nodei = ni;
        end

        %-----------------------------------------------------------------------
        % Set final node handle
        function obj = set.nodef(obj,nf)
            obj.nodef = nf;
        end

        %-----------------------------------------------------------------------
        % Set initial node hinge flag
        function obj = set.hingei(obj,hi)
            obj.hingei = hi;
        end

        %-----------------------------------------------------------------------
        % Set final node hinge flag
        function obj = set.hingef(obj,hf)
            obj.hingef = hf;
        end

        %-----------------------------------------------------------------------
        % Set element length
        function obj = set.length(obj,L)
            obj.length = L;
        end

        %-----------------------------------------------------------------------
        % Set element orientation cosine X
        function obj = set.cosine_X(obj,cx)
            obj.cosine_X = cx;
        end

        %-----------------------------------------------------------------------
        % Set element orientation cosine Y
        function obj = set.cosine_Y(obj,cy)
            obj.cosine_Y = cy;
        end

        %-----------------------------------------------------------------------
        % Set element orientation cosine Z
        function obj = set.cosine_Z(obj,cz)
            obj.cosine_Z = cz;
        end

        %-----------------------------------------------------------------------
        % Set element X component of auxiliary vector in local XZ plane
        function obj = set.vz_X(obj,vzx)
            obj.vz_X = vzx;
        end

        %-----------------------------------------------------------------------
        % Set element Y component of auxiliary vector in local XZ plane
        function obj = set.vz_Y(obj,vzy)
            obj.vz_Y = vzy;
        end

        %-----------------------------------------------------------------------
        % Set element Z component of auxiliary vector in local XZ plane
        function obj = set.vz_Z(obj,vzz)
            obj.vz_Z = vzz;
        end

        %-----------------------------------------------------------------------
        % Set element rotation transformation matrix
        function obj = set.rot(obj,rot)
            obj.rot = rot;
        end

        %-----------------------------------------------------------------------
        % Set element gather vector
        function obj = set.gle(obj,gle)
            obj.gle = gle;
        end

        %-----------------------------------------------------------------------
        % Set vector of uniformly distributed load components
        function obj = set.uniformLoad(obj,unif)
            obj.uniformLoad = unif;
        end

        %-----------------------------------------------------------------------
        % Set flag for uniform load direction
        function obj = set.uniformDirection(obj,dir)
            obj.uniformDirection = dir;
        end

        %-----------------------------------------------------------------------
        % Set vector of linearly distributed load components
        function obj = set.linearLoad(obj,linear)
            obj.linearLoad = linear;
        end

        %-----------------------------------------------------------------------
        % Set flag for linear load direction
        function obj = set.linearDirection(obj,dir)
            obj.linearDirection = dir;
        end

        %-----------------------------------------------------------------------
        % Set flag for linear load direction
        function obj = set.tempVar_GC(obj,tvgc)
            obj.tempVar_GC = tvgc;
        end

        %-----------------------------------------------------------------------
        % Set flag for linear load direction
        function obj = set.tempVar_Y(obj,tvy)
            obj.tempVar_Y = tvy;
        end

        %-----------------------------------------------------------------------
        % Set flag for linear load direction
        function obj = set.tempVar_Z(obj,tvz)
            obj.tempVar_Z = tvz;
        end

        %-----------------------------------------------------------------------
        % Set array of axial internal forces at member ends
        function obj = set.axial_force(obj,af)
            obj.axial_force = af;
        end

        %-----------------------------------------------------------------------
        % Set array of shear internal forces at member ends
        function obj = set.shear_force_Y(obj,sfy)
            obj.shear_force_Y = sfy;
        end

        %-----------------------------------------------------------------------
        % Set array of shear internal forces at member ends
        function obj = set.shear_force_Z(obj,sfz)
            obj.shear_force_Z = sfz;
        end

        %-----------------------------------------------------------------------
        % Set array of bending internal moments at member ends
        function obj = set.bending_moment_Y(obj,bmy)
            obj.bending_moment_Y = bmy;
        end

        %-----------------------------------------------------------------------
        % Set array of bending internal moments at member ends
        function obj = set.bending_moment_Z(obj,bmz)
            obj.bending_moment_Z = bmz;
        end

        %-----------------------------------------------------------------------
        % Set array of torsion internal moments at member ends
        function obj = set.torsion_moment(obj,tm)
            obj.torsion_moment = tm;
        end
    end

    %% Public get methods
    methods
        %-----------------------------------------------------------------------
        % Get analysis model handle
        function anm = get.anm(obj)
            anm = obj.anm;
        end

        %-----------------------------------------------------------------------
        % Get element type parameter value
        function type = get.type(obj)
            type = obj.type;
        end

        %-----------------------------------------------------------------------
        % Get material handle
        function mat = get.material(obj)
            mat = obj.material;
        end

        %-----------------------------------------------------------------------
        % Get cross section handle
        function sec = get.section(obj)
            sec = obj.section;
        end

        %-----------------------------------------------------------------------
        % Get initial node handle
        function ni = get.nodei(obj)
            ni = obj.nodei;
        end

        %-----------------------------------------------------------------------
        % Get final node handle
        function nf = get.nodef(obj)
            nf = obj.nodef;
        end

        %-----------------------------------------------------------------------
        % Get initial node hinge flag
        function hi = get.hingei(obj)
            hi = obj.hingei;
        end

        %-----------------------------------------------------------------------
        % Get final node hinge flag
        function hf = get.hingef(obj)
            hf = obj.hingef;
        end

        %-----------------------------------------------------------------------
        % Get element length
        function L = get.length(obj)
            L = obj.length;
        end

        %-----------------------------------------------------------------------
        % Get element orientation cosine x
        function cx = get.cosine_X(obj)
            cx = obj.cosine_X;
        end

        %-----------------------------------------------------------------------
        % Get element orientation cosine Y
        function cy = get.cosine_Y(obj)
            cy = obj.cosine_Y;
        end

        %-----------------------------------------------------------------------
        % Get element orientation cosine Z
        function cz = get.cosine_Z(obj)
            cz = obj.cosine_Z;
        end

        %-----------------------------------------------------------------------
        % Get element X component of auxiliary vector in local XZ plane
        function vzx = get.vz_X(obj)
            vzx = obj.vz_X;
        end

        %-----------------------------------------------------------------------
        % Get element Y component of auxiliary vector in local XZ plane
        function vzy = get.vz_Y(obj)
            vzy = obj.vz_Y;
        end

        %-----------------------------------------------------------------------
        % Get element Z component of auxiliary vector in local XZ plane
        function vzz = get.vz_Z(obj)
            vzz = obj.vz_Z;
        end

        %-----------------------------------------------------------------------
        % Get element rotation transformation matrix
        function rot = get.rot(obj)
            rot = obj.rot;
        end

        %-----------------------------------------------------------------------
        % Get element gather vector
        function gle = get.gle(obj)
            gle = obj.gle;
        end

        %-----------------------------------------------------------------------
        % Get vector of uniformly distributed load components
        function unif = get.uniformLoad(obj)
            unif = obj.uniformLoad;
        end

        %-----------------------------------------------------------------------
        % Get flag for uniform load direction
        function dir = get.uniformDirection(obj)
            dir = obj.uniformDirection;
        end

        %-----------------------------------------------------------------------
        % Get vector of linearly distributed load components
        function linear = get.linearLoad(obj)
            linear = obj.linearLoad;
        end

        %-----------------------------------------------------------------------
        % Get flag for linear load direction
        function dir = get.linearDirection(obj)
            dir = obj.linearDirection;
        end

        %-----------------------------------------------------------------------
        % Get flag for linear load direction
        function tvgc = get.tempVar_GC(obj)
            tvgc = obj.tempVar_GC;
        end

        %-----------------------------------------------------------------------
        % Get flag for linear load direction
        function tvy = get.tempVar_Y(obj)
            tvy = obj.tempVar_Y;
        end

        %-----------------------------------------------------------------------
        % Get flag for linear load direction
        function tvx = get.tempVar_Z(obj)
            tvx = obj.tempVar_Z;
        end

        %-----------------------------------------------------------------------
        % Get array of axial internal forces at member ends
        function af = get.axial_force(obj)
            af = obj.axial_force;
        end

        %-----------------------------------------------------------------------
        % Get array of shear internal forces at member ends
        function sfy = get.shear_force_Y(obj)
            sfy = obj.shear_force_Y;
        end

        %-----------------------------------------------------------------------
        % Get array of shear internal forces at member ends
        function sfz = get.shear_force_Z(obj)
            sfz = obj.shear_force_Z;
        end

        %-----------------------------------------------------------------------
        % Get array of bending internal moments at member ends
        function bmy = get.bending_moment_Y(obj)
            bmy = obj.bending_moment_Y;
        end

        %-----------------------------------------------------------------------
        % Get array of bending internal moments at member ends
        function bmz = get.bending_moment_Z(obj)
            bmz = obj.bending_moment_Z;
        end

        %-----------------------------------------------------------------------
        % Get array of torsion internal moments at member ends
        function tm = get.torsion_moment(obj)
            tm = obj.torsion_moment;
        end
    end

    %% Abstract methods
    methods (Abstract)
        %-----------------------------------------------------------------------
        % Generate member flexural stiffness coefficient matrix in local
        % XY plane for a given element.
        % Output argument:
        %  kef: a 4x4 matrix with flexural stiffness coefficients
        kef = flexuralStiffCoeff_XY(obj)

        %-----------------------------------------------------------------------
        % Generate member flexural stiffness coefficient matrix in local
        % XZ plane for a given element.
        % Output argument:
        %  kef: a 4x4 matrix with flexural stiffness coefficients
        kef = flexuralStiffCoeff_XZ(obj)

        %-----------------------------------------------------------------------
        % Generate member flexural fixed end force vector in local XY 
        % plane for a linearly distributed load applied to given element.
        % Input arguments:
        %  qyi: transversal load value at init node
        %  qyf: transversal load value at end (final) node
        % Output argument:
        %  fef: a 4x1 vector with flexural (transversal) fixed end forces
        fef = flexuralLinearLoadFEF_XY(obj,qyi,qyf)

        %-----------------------------------------------------------------------
        % Generate member flexural fixed end force vector in local XZ 
        % plane for a linearly distributed load applied to given element.
        % Input arguments:
        %  qzi: transversal load value at init node
        %  qzf: transversal load value at end (final) node
        % Output argument:
        %  fef: a 4x1 vector with flexural (transversal) fixed end forces
        fef = flexuralLinearLoadFEF_XZ(obj,qzi,qzf)

        %-----------------------------------------------------------------------
        % Generate member flexural fixed end force vector in local XY 
        % plane for a temperature variation of a given element.
        % Output argument:
        %  fef: a 4x1 vector with flexural (transversal) fixed end forces
        fef = flexuralThermalLoadFEF_XY(obj)

        %-----------------------------------------------------------------------
        % Generate member flexural fixed end force vector in local XZ 
        % plane for a temperature variation of a given element.
        % Output argument:
        %  fef: a 4x1 vector with flexural (transversal) fixed end forces
        fef = flexuralThermalLoadFEF_XZ(obj)
    end

    %% Public methods
    methods
        %-----------------------------------------------------------------------
        % Generate stiffness matrix in global system for a given element.
        % Compute element (member) stiffness matrix in local system and
        % transform element stiffness matrix from local to global system.
        % Output:
        %  keg: element stiffness matrix in global system
        function keg = gblStiffMtx(obj)
            % Compute element (member) stiffness matrix in local system
            kel = obj.anm.elemLocStiffMtx(obj);

            % Transform element stiffness matrix from local to global system
%   *** COMPLETE HERE - Elem-01 ***

%   *** COMPLETE HERE - Elem-01 ***
        end

        %-----------------------------------------------------------------------
        % Generate member axial stiffness coefficient matrix for a
        % given element.
        % Output:
        % kea: a 2x2 matrix with axial stiffness coefficients
        function kea = axialStiffCoeff(obj)
            E = obj.material.elasticity;   % get element elasticity coeff.
            A = obj.section.area_X;        % get element full (axial) cross section area
            L = obj.length;                % element length

%   *** COMPLETE HERE - Elem-02 ***


%   *** COMPLETE HERE - Elem-02 ***
        end

        %-----------------------------------------------------------------------
        % Generate member torsion stiffness coefficient matrix for a
        % given element.
        % Output:
        %  ket: a 2x2 matrix with torsion stiffness coefficients
        function ket = torsionStiffCoeff(obj)
            include_constants;

            G = obj.material.shear;            % get element shear coefficient
            Jt = obj.section.inertia_X;        % get element torsion mom. of inertia coeff.
            L = obj.length;                    % element length

            if((obj.hingei == CONTINUOUS_END) && (obj.hingef == CONTINUOUS_END))

%   *** COMPLETE HERE - Elem-03 ***


%   *** COMPLETE HERE - Elem-03 ***

            end
        end

        %-----------------------------------------------------------------------
        % Generate equivalent nodal load (ENL) vector in global system for
        % uniformely distributed load applied to a given element.
        % Compute element fixed end forces (FEF) in local system for a uniformely
        % distributed load applied on a given element and transform element 
        % fixed end forces in local system to equivalent nodal loads in
        % global system.
        % Output:
        %  feg:  equivalent nodal load vector in global system
        function feg = gblUniformLoadENL(obj)
            % Compute element fixed end forces (FEF) in local system for a uniformely
            % distributed load applied on a given element
            fel = obj.anm.elemLocUniformLoadFEF(obj);

            % Transform element fixed end forces in local system to equivalent nodal
            % loads in global system
%   *** COMPLETE HERE - Elem-04 ***

%   *** COMPLETE HERE - Elem-04 ***
        end

        %-----------------------------------------------------------------------
        % Generate equivalent nodal load (ENL) vector in global system for
        % linearly distributed load applied to a given element.
        % Compute element fixed end forces (FEF) in local system for a linearly
        % distributed load applied on a given element and transform element 
        % fixed end forces in local system to equivalent nodal loads in
        % global system.
        % Output:
        %  feg:  equivalent nodal load vector in global system
        function feg = gblLinearLoadENL(obj)
            % Compute element fixed end forces (FEF) in local system for a linearly
            % distributed load applied on a given element
            fel = obj.anm.elemLocLinearLoadFEF(obj);

            % Transform element fixed end forces in local system to equivalent nodal
            % loads in global system
%   *** COMPLETE HERE - Elem-05 ***

%   *** COMPLETE HERE - Elem-05 ***
        end

        %-----------------------------------------------------------------------
        % Generate equivalent nodal load (ENL) vector in global system for a
        % temperature variation of a given element.
        % Compute element fixed end forces (FEF) in local system for a thermal 
        % expansion of a given element and transform element fixed end forces 
        % in local system to equivalent nodal loads in global system.
        % Output:
        %  feg:  equivalent nodal load vector in global system
        function feg = gblThermalLoadENL(obj)
            % Compute element fixed end forces (FEF) in local system for a thermal
            % expansion of a given element
            fel = obj.anm.elemLocThermalLoadFEF(obj);

            % Transform element fixed end forces in local system to equivalent nodal
            % loads in global system
%   *** COMPLETE HERE - Elem-06 ***

%   *** COMPLETE HERE - Elem-06 ***
        end

        %-----------------------------------------------------------------------
        % Generate member axial fixed end force vector for a linearly distributed
        % load applied to given element.
        % Output:
        %  fea: a 2x1 vector with axial fixed end force components
        % Input arguments:
        %  qxi: axial load value at init node
        %  qxf: axial load value at end (final) node
        function fea = axialLinearLoadFEF(obj,qxi,qxf)
            L = obj.length;   % member length

%   *** COMPLETE HERE - Elem-07 ***


%   *** COMPLETE HERE - Elem-07 ***
        end

        %-----------------------------------------------------------------------
        % Generate member axial fixed end force vector for a temperature variation
        % of a given element.
        % Output:
        %  fea: a 2x1 vector with axial fixed end force components
        function fea = axialThermalLoadFEF(obj)
            E = obj.material.elasticity;
            alpha = obj.material.thermexp;
            A = obj.section.area_X;

            EA = E*A;

%   *** COMPLETE HERE - Elem-08 ***


%   *** COMPLETE HERE - Elem-08 ***
        end

        %-----------------------------------------------------------------------
        % Generate member transversal fixed end force vector for a linearly distributed
        % load applied to given element assuming that it is simply support on both ends.
        % Output:
        %  fes: a 2x1 vector with transversal simply supported fixed end forces:
        %   fes(1,1) -> transversal force at init node
        %   fes(2,1) -> transversal force at end node
        % Input arguments:
        %  qyi: transversal load value at init node
        %  qyf: transversal load value at end (final) node
        function fes = simplySuppLinearLoadFEF(obj,qyi,qyf)
            L = obj.length;     % member length

            q0 = qyi;           % uniform contribution of load
            q1 = qyf - qyi;     % linear contribution of load

%   *** COMPLETE HERE - Elem-09 ***


%   *** COMPLETE HERE - Elem-09 ***
        end

        %-----------------------------------------------------------------------
        % Get internal force vector in local system at the end nodes of a given
        % element for the global analysis from nodal displacements and
        % rotations.
        % Extract solution at element nodes (displacements and rotations at
        % member end nodes in global system).
        % Transform solution at element nodes from global system to local
        % system.
        % Compute element (member) stiffness matrix in local system.
        % Compute internal force vector in local system at end nodes of
        % member.
        % Input arguments:
        %  drv: handle to analysis driver object
        % Output:
        %  fel: internal force vector in local system at end nodes of member
        function fel = gblAnlIntForce(obj,drv)
            % Extract solution at element nodes (displacements and rotations at
            % member end nodes in global system)
            dg = drv.D(obj.gle);

            % Transform solution at element nodes from global system to local system
            dl = %   *** COMPLETE HERE - Elem-10 ***

            % Compute element (member) stiffness matrix in local system
            kel = %   *** COMPLETE HERE - Elem-11 ***

            % Compute internal force vector in local system at end nodes of member
%   *** COMPLETE HERE - Elem-12 ***

%   *** COMPLETE HERE - Elem-12 ***
        end

        %-----------------------------------------------------------------------
        % Cleans data structure of a elem object.
        function obj = clean(obj)
            obj.nen = 2;
            obj.anm = [];
            obj.type = 0;
            obj.material = [];
            obj.section = [];
            obj.nodei = [];
            obj.nodef = [];
            obj.hingei = 0;
            obj.hingef = 0;
            obj.length = 0;
            obj.cosine_X = 0;
            obj.cosine_Y = 0;
            obj.cosine_Z = 0;
            obj.vz_X = 0;
            obj.vz_Y = 0;
            obj.vz_Z = 0;
            obj.rot = [];
            obj.gle = [];
            obj.uniformLoad = [];
            obj.uniformDirection = 0;
            obj.linearLoad = [];
            obj.linearDirection = 0;
            obj.tempVar_GC = 0;
            obj.tempVar_Y = 0;
            obj.tempVar_Z = 0;
            obj.axial_force = [];
            obj.shear_force_Y = [];
            obj.shear_force_Z = [];
            obj.bending_moment_Y = [];
            obj.bending_moment_Z = [];
            obj.torsion_moment = [];
        end
    end
end