%% Elem_Navier class
%
% This is a sub-class, in the Object Oriented Programming (OOP) paradigm,
% of super-class <elem.html *Elem*> in the <main.html LESM (Linear Elements
% Structure Model)> program. This sub-class implements abstract methods, 
% defined in super-class *Elem*, that deal with Navier (Euler-Bernoulli)
% flexural behavior of linear structure elements.  In Euler-Bernoulli
% flexural behavior, it is assumed that there is no shear deformation. As a
% consequence, bending of a linear structure element is such that its
% cross-section remains plane and normal to the element longitudinal
% axis.
%
%% Authors
% Luiz Fernando Martha and Rafael Lopez Rangel
%
%% History
% @version 1.00
%
% Initial version: August-December 2016
%%%
% Prepared for the course ENG 1240 - Anlise de Estruturas III, 2016.2,
% Department of Civil Engineering, PUC-Rio.
%
classdef Elem_Navier < Elem
    %%
    % <elem.html See documentation on *Elem* super-class>.

    %% Constructor method
    methods
        %-----------------------------------------------------------------------
        function obj = Elem_Navier(anm,type,mat,sec,ni,nf,hi,hf,L,cx,cy,cz,...
                                   rot,gle,unif,dirunif,lin,dirlin,...
                                   tvgc,tvy,tvz,af,sfy,sfz,bmy,bmz,tm)
            if (nargin > 0)
                obj.anm = anm;
                obj.type = type;
                obj.material = mat;
                obj.section = sec;
                obj.nodei = ni;
                obj.nodef = nf;
                obj.hingei = hi;
                obj.hingef = hf;
                obj.length = L;
                obj.cosine_X = cx;
                obj.cosine_Y = cy;
                obj.cosine_Z = cz;
                obj.rot = rot;
                obj.gle = gle;
                obj.uniformLoad = unif;
                obj.uniformDirection = dirunif;
                obj.linearLoad = lin;
                obj.linearDirection = dirlin;
                obj.tempVar_GC = tvgc;
                obj.shear_force_Y = tvy;
                obj.shear_force_Z = tvz;
                obj.axial_force = af;
                obj.shear_force_Y = sfy;
                obj.shear_force_Z = sfz;
                obj.bending_moment_Y = bmy;
                obj.bending_moment_Z = bmz;
                obj.torsion_moment = tm;
            end
        end
    end

    %% Public methods
    % Implementation of the abstract methods declared in super-class <elem.html *Elem*>.
    methods
        %-----------------------------------------------------------------------
        % Generate member flexural stiffness coefficient matrix in local XY plane
        % for a given element.
        % Output:
        %  kef: a 4x4 matrix with flexural stiffness coefficients:
        %      D -> transversal displacement
        %      R -> rotation
        %      ini -> initial node
        %      end -> end node
        %   kef(1,1) = k_Dini_Dini
        %   kef(1,2) = k_Dini_Rini
        %   kef(1,3) = k_Dini_Dend
        %   kef(1,4) = k_Dini_Rend
        %   kef(2,1) = k_Rini_Dini = k_Dini_Rini
        %   kef(2,2) = k_Rini_Rini
        %   kef(2,3) = k_Rini_Dend
        %   kef(2,4) = k_Rini_Rend
        %   kef(3,1) = k_Dend_Dini = k_Dini_Dend
        %   kef(3,2) = k_Dend_Rini = k_Rini_Dend
        %   kef(3,3) = k_Dend_Dend
        %   kef(3,4) = k_Dend_Rend
        %   kef(4,1) = k_Rend_Dini = k_Dini_Rend
        %   kef(4,2) = k_Rend_Rini = k_Rini_Rend
        %   kef(4,3) = k_Rend_Dend = k_Dend_Rend
        %   kef(4,4) = k_Rend_Rend
        function kef = flexuralStiffCoeff_XY(obj)
            include_constants;
            
            E = obj.material.elasticity;       % get element elasticity coeff.
            I = obj.section.inertia_Z;         % get element bending mom.of inertia coeff.
            L = obj.length;                    % element length

            L2 = L^2;                          % square of length
            L3 = L2*L;                         % length to cubic power
            EI = E*I;                          % flexural rigidity

            % Assemble matrix that stores member axial stiffness coefficients

            if((obj.hingei == CONTINUOUS_END) && (obj.hingef == CONTINUOUS_END))

%   *** COMPLETE HERE - Elem_Navier-01 ***




%   *** COMPLETE HERE - Elem_Navier-01 ***

            elseif((obj.hingei == HINGED_END) && (obj.hingef == CONTINUOUS_END))

%   *** COMPLETE HERE - Elem_Navier-02 ***




%   *** COMPLETE HERE - Elem_Navier-02 ***

            elseif((obj.hingei == CONTINUOUS_END) && (obj.hingef == HINGED_END))

%   *** COMPLETE HERE - Elem_Navier-03 ***




%   *** COMPLETE HERE - Elem_Navier-03 ***

            elseif((obj.hingei == HINGED_END) && (obj.hingef == HINGED_END))

%   *** COMPLETE HERE - Elem_Navier-04 ***




%   *** COMPLETE HERE - Elem_Navier-04 ***

            end
        end

        %-----------------------------------------------------------------------
        % Generate member flexural stiffness coefficient matrix in local XZ plane
        % for a given element.
        % Output:
        %  kef: a 4x4 matrix with flexural stiffness coefficients:
        %      D -> transversal displacement
        %      R -> rotation
        %      ini -> initial node
        %      end -> end node
        %   kef(1,1) = k_Dini_Dini
        %   kef(1,2) = k_Dini_Rini
        %   kef(1,3) = k_Dini_Dend
        %   kef(1,4) = k_Dini_Rend
        %   kef(2,1) = k_Rini_Dini = k_Dini_Rini
        %   kef(2,2) = k_Rini_Rini
        %   kef(2,3) = k_Rini_Dend
        %   kef(2,4) = k_Rini_Rend
        %   kef(3,1) = k_Dend_Dini = k_Dini_Dend
        %   kef(3,2) = k_Dend_Rini = k_Rini_Dend
        %   kef(3,3) = k_Dend_Dend
        %   kef(3,4) = k_Dend_Rend
        %   kef(4,1) = k_Rend_Dini = k_Dini_Rend
        %   kef(4,2) = k_Rend_Rini = k_Rini_Rend
        %   kef(4,3) = k_Rend_Dend = k_Dend_Rend
        %   kef(4,4) = k_Rend_Rend
        function kef = flexuralStiffCoeff_XZ(obj)
            include_constants;
            
            E = obj.material.elasticity;       % get element elasticity coeff.
            I  = obj.section.inertia_Y;        % get element bending mom.of inertia coeff.
            L = obj.length;                    % element length

            L2 = L^2;                          % square of length
            L3 = L2*L;                         % length to cubic power
            EI  = E*I;                         % flexural rigidity

            % Assemble matrix that stores member axial stiffness coefficients

            if((obj.hingei == CONTINUOUS_END) && (obj.hingef == CONTINUOUS_END))

%   *** COMPLETE HERE - Elem_Navier-05 ***




%   *** COMPLETE HERE - Elem_Navier-05 ***

            elseif((obj.hingei == HINGED_END) && (obj.hingef == CONTINUOUS_END))

%   *** COMPLETE HERE - Elem_Navier-06 ***




%   *** COMPLETE HERE - Elem_Navier-06 ***

            elseif((obj.hingei == CONTINUOUS_END) && (obj.hingef == HINGED_END))

%   *** COMPLETE HERE - Elem_Navier-07 ***




%   *** COMPLETE HERE - Elem_Navier-07 ***

            elseif((obj.hingei == HINGED_END) && (obj.hingef == HINGED_END))

%   *** COMPLETE HERE - Elem_Navier-08 ***




%   *** COMPLETE HERE - Elem_Navier-08 ***

            end
        end

        %-----------------------------------------------------------------------
        % Generate member flexural fixed end force vector in local XY plane
        % for a linearly distributed load applied to given element.
        % Input arguments:
        %  qyi: transversal load value at init node
        %  qyf: transversal load value at end (final) node
        % Output:
        %  fef: a 4x1 vector with flexural (transversal) fixed end forces:
        %      D -> transversal displacement
        %      R -> rotation
        %      ini -> initial node
        %      end -> end node
        %   fef(1,1) = f_Dini
        %   fef(2,1) = f_Rini
        %   fef(3,1) = f_Dend
        %   fef(4,1) = f_Rend
        function fef = flexuralLinearLoadFEF_XY(obj,qyi,qyf)
            include_constants;

            L = obj.length;     % element length
            L2 = L^2;           % square of length
            q0 = qyi;           % uniform contribution of load
            q1 = qyf - qyi;     % linear contribution of load

            VA = -(q0*L/2.0   + q1*3.0*L/20.0);
            MA = -(q0*L2/12.0 + q1*L2/30.0);
            VB = -(q0*L/2.0   + q1*7.0*L/20.0);
            MB =  (q0*L2/12.0 + q1*L2/20.0);

            tAB =  0.5;
            tBA =  tAB;

            % Assemble vector that stores fixed end forces

            if((obj.hingei == CONTINUOUS_END) && (obj.hingef == CONTINUOUS_END))

%   *** COMPLETE HERE - Elem_Navier-09 ***




%   *** COMPLETE HERE - Elem_Navier-09 ***

            elseif((obj.hingei == HINGED_END) && (obj.hingef == CONTINUOUS_END))

%   *** COMPLETE HERE - Elem_Navier-10 ***




%   *** COMPLETE HERE - Elem_Navier-10 ***

            elseif((obj.hingei == CONTINUOUS_END) && (obj.hingef == HINGED_END))

%   *** COMPLETE HERE - Elem_Navier-11 ***




%   *** COMPLETE HERE - Elem_Navier-11 ***

            elseif((obj.hingei == HINGED_END) && (obj.hingef == HINGED_END))

%   *** COMPLETE HERE - Elem_Navier-12 ***




%   *** COMPLETE HERE - Elem_Navier-12 ***

            end
        end

        %-----------------------------------------------------------------------
        % Generate member flexural fixed end force vector in local XZ plane
        % for a linearly distributed load applied to given element.
        % Input arguments:
        %  qzi: transversal load value at init node
        %  qzf: transversal load value at end (final) node
        % Output:
        %  fef: a 4x1 vector with flexural (transversal) fixed end forces:
        %      D -> transversal displacement
        %      R -> rotation
        %      ini -> initial node
        %      end -> end node
        %   fef(1,1) = f_Dini
        %   fef(2,1) = f_Rini
        %   fef(3,1) = f_Dend
        %   fef(4,1) = f_Rend
        function fef = flexuralLinearLoadFEF_XZ(obj,qzi,qzf)
            include_constants;

            L = obj.length;     % element length
            L2 = L^2;           % square of length
            q0 = qzi;           % uniform contribution of load
            q1 = qzf - qzi;     % linear contribution of load

            VA = -(q0*L/2.0   + q1*3.0*L/20.0);
            MA =  (q0*L2/12.0 + q1*L2/30.0);
            VB = -(q0*L/2.0   + q1*7.0*L/20.0);
            MB = -(q0*L2/12.0 + q1*L2/20.0);

            tAB =  0.5;
            tBA =  tAB;

            % Assemble vector that stores fixed end forces

            if((obj.hingei == CONTINUOUS_END) && (obj.hingef == CONTINUOUS_END))

%   *** COMPLETE HERE - Elem_Navier-13 ***




%   *** COMPLETE HERE - Elem_Navier-13 ***

            elseif((obj.hingei == HINGED_END) && (obj.hingef == CONTINUOUS_END))

%   *** COMPLETE HERE - Elem_Navier-14 ***




%   *** COMPLETE HERE - Elem_Navier-14 ***

            elseif((obj.hingei == CONTINUOUS_END) && (obj.hingef == HINGED_END))

%   *** COMPLETE HERE - Elem_Navier-15 ***




%   *** COMPLETE HERE - Elem_Navier-15 ***

            elseif((obj.hingei == HINGED_END) && (obj.hingef == HINGED_END))

%   *** COMPLETE HERE - Elem_Navier-16 ***




%   *** COMPLETE HERE - Elem_Navier-16 ***

            end
        end

        %-----------------------------------------------------------------------
        % Implementation of an abstract method declared in super-class Elem:
        % Generate member flexural fixed end force vector in local XY 
        % plane for a thermal expansion of a given element.
        % Output argument:
        %  fef: a 4x1 vector with flexural (transversal) fixed end forces:
        %      D -> transversal displacement
        %      R -> rotation
        %      ini -> initial node
        %      end -> end node
        %   fef(1,1) = f_Dini
        %   fef(2,1) = f_Rini
        %   fef(3,1) = f_Dend
        %   fef(4,1) = f_Rend
        function fef = flexuralThermalLoadFEF_XY(obj)
            include_constants;

            E = obj.material.elasticity;
            alpha = obj.material.thermexp;
            I = obj.section.inertia_Z;
            h = obj.section.height_Y;
            dty = obj.tempVar_Y;

            EI = E*I;

            MA = (EI*alpha*dty)/h;
            MB = -(EI*alpha*dty)/h;
            tAB = 0.5;
            tBA = tAB;

            % Assemble vector that stores member thermal fixed end forces

            if((obj.hingei == CONTINUOUS_END) && (obj.hingef == CONTINUOUS_END))

%   *** COMPLETE HERE - Elem_Navier-17 ***




%   *** COMPLETE HERE - Elem_Navier-17 ***

            elseif((obj.hingei == HINGED_END) && (obj.hingef == CONTINUOUS_END))

%   *** COMPLETE HERE - Elem_Navier-18 ***




%   *** COMPLETE HERE - Elem_Navier-18 ***

            elseif((obj.hingei == CONTINUOUS_END) && (obj.hingef == HINGED_END))

%   *** COMPLETE HERE - Elem_Navier-19 ***




%   *** COMPLETE HERE - Elem_Navier-19 ***

            elseif((obj.hingei == HINGED_END) && (obj.hingef == HINGED_END))

%   *** COMPLETE HERE - Elem_Navier-20 ***




%   *** COMPLETE HERE - Elem_Navier-20 ***

            end
        end

        %-----------------------------------------------------------------------
        % Generate member flexural fixed end force vector in local XZ plane
        % for a thermal expansion of a given element.
        % Output:
        %  fef: a 4x1 vector with flexural (transversal) fixed end forces:
        %      D -> transversal displacement
        %      R -> rotation
        %      ini -> initial node
        %      end -> end node
        %   fef(1,1) = f_Dini
        %   fef(2,1) = f_Rini
        %   fef(3,1) = f_Dend
        %   fef(4,1) = f_Rend
        function fef = flexuralThermalLoadFEF_XZ(obj)
            include_constants;

            E = obj.material.elasticity;
            alpha = obj.material.thermexp;
            I = obj.section.inertia_Y;
            h = obj.section.height_Z;
            dtz = obj.tempVar_Z;

            EI = E*I;

            MA = (EI*alpha*dtz)/h;
            MB = -(EI*alpha*dtz)/h;
            tAB = 0.5;
            tBA = tAB;

            % Assemble vector that stores member thermal fixed end forces

            if((obj.hingei == CONTINUOUS_END) && (obj.hingef == CONTINUOUS_END))

%   *** COMPLETE HERE - Elem_Navier-21 ***




%   *** COMPLETE HERE - Elem_Navier-21 ***

            elseif((obj.hingei == HINGED_END) && (obj.hingef == CONTINUOUS_END))

%   *** COMPLETE HERE - Elem_Navier-22 ***




%   *** COMPLETE HERE - Elem_Navier-22 ***

            elseif((obj.hingei == CONTINUOUS_END) && (obj.hingef == HINGED_END))

%   *** COMPLETE HERE - Elem_Navier-23 ***




%   *** COMPLETE HERE - Elem_Navier-23 ***

            elseif((obj.hingei == HINGED_END) && (obj.hingef == HINGED_END))

%   *** COMPLETE HERE - Elem_Navier-24 ***




%   *** COMPLETE HERE - Elem_Navier-24 ***

            end
        end
    end
end