%% Node class
%
% This is a class in the Object Oriented Programming (OOP) paradigm
% that defines node objects in the <main.html LESM (Linear Elements
% Structure Model)>) program.
%
%% Authors
% Luiz Fernando Martha and Rafael Lopez Rangel
%
%% History
% @version 1.00
%
% Initial version: August-December 2016
%%%
% Prepared for the course ENG 1240 - Anlise de Estruturas III, 2016.2,
% Department of Civil Engineering, PUC-Rio.
%
classdef Node < hgsetget
    %%
    % <http://www.mathworks.com/help/matlab/handle-classes.html
    % See documentation on *hgsetget* handle super-class>.

    %% Public attributes
    properties (SetAccess = public, GetAccess = public)
        id = 0;            % node identification number
        coord_X = 0;       % nodal coordinate on global axis x
        coord_Y = 0;       % nodal coordinate on global axis y
        coord_Z = 0;       % nodal coordinate on global axis z
        ebc = [];          % vector of nodal essential boundary condition flags
        prescDispl = [];   % vector of nodal prescribed displacement values
        nodalLoad = [];    % vector of nodal load components
    end

    %% Constructor method
    methods
        %-----------------------------------------------------------------------
        function obj = Node(id,x,y,z,ebc,pd,nl)
            if (nargin > 0)
                obj.id = id;
                obj.coord_X = x;
                obj.coord_Y = y;
                obj.coord_Z = z;
                obj.ebc = ebc;
                obj.prescDispl = pd;
                obj.nodalLoad = nl;
            end
        end
    end

    %% Public set methods
    methods
        %-----------------------------------------------------------------------
        % Set node id number
        function obj = set.id(obj,id)
            obj.id = id;
        end

        %-----------------------------------------------------------------------
        % Set nodal x coordinate value
        function obj = set.coord_X(obj,x)
            obj.coord_X = x;
        end

        %-----------------------------------------------------------------------
        % Set nodal y coordinate value
        function obj = set.coord_Y(obj,y)
            obj.coord_Y = y;
        end

        %-----------------------------------------------------------------------
        % Set nodal z coordinate value
        function obj = set.coord_Z(obj,z)
            obj.coord_Z = z;
        end

        %-----------------------------------------------------------------------
        % Set vector of nodal boundary condition flags
        function obj = set.ebc(obj,ebc)
            obj.ebc = ebc;
        end

        %-----------------------------------------------------------------------
        % Set vector of nodal prescribed displacement values
        function obj = set.prescDispl(obj,pd)
            obj.prescDispl = pd;
        end

        %-----------------------------------------------------------------------
        % Set vector of nodal load components
        function obj = set.nodalLoad(obj,nl)
            obj.nodalLoad = nl;
        end
    end

    %% Public get methods
    methods
        %-----------------------------------------------------------------------
        % Get node id number
        function id = get.id(obj)
            id = obj.id;
        end

        %-----------------------------------------------------------------------
        % Get nodal x coordinate value
        function x = get.coord_X(obj)
            x = obj.coord_X;
        end

        %-----------------------------------------------------------------------
        % Get nodal y coordinate value
        function y = get.coord_Y(obj)
            y = obj.coord_Y;
        end

        %-----------------------------------------------------------------------
        % Get nodal z coordinate value
        function z = get.coord_Z(obj)
            z = obj.coord_Z;
        end

        %-----------------------------------------------------------------------
        % Get vector of nodal boundary condition flags
        function ebc = get.ebc(obj)
            ebc = obj.ebc;
        end

        %-----------------------------------------------------------------------
        % Get vector of nodal prescribed displacement values
        function pd = get.prescDispl(obj)
            pd = obj.prescDispl;
        end

        %-----------------------------------------------------------------------
        % Get vector of nodal load components
        function nl = get.nodalLoad(obj)
            nl = obj.nodalLoad;
        end
    end

    %% Public methods
    methods
        %---------------------------------------------------------------------------
        % Count total number of elements and hinged elements connected to a node.
        % Output:
        %  tot: total number of elements connected to a node
        %  he:  total number os hinged elements connected to a node
        % Input arguments:
        %  drv: handle to analysis driver object
        function [tot,he] = elemsIncidence(obj,drv)
            tot = 0;
            he = 0;
            
            for e = 1:drv.nel
                if drv.elems(e).nodei.id == obj.id
                    tot = tot + 1;
                    if drv.elems(e).hingei == 0
                        he = he + 1;
                    end
                    
                elseif drv.elems(e).nodef.id == obj.id
                    tot = tot + 1;
                    if drv.elems(e).hingef == 0
                        he = he + 1;
                    end
                end
            end
        end
        
        %-----------------------------------------------------------------------
        % Cleans data structure of a node object
        function obj = clean(obj)
            obj.id = 0;
            obj.coord_X = 0;
            obj.coord_Y = 0;
            obj.coord_Z = 0;
            obj.ebc = [];
            obj.prescDispl = [];
            obj.nodalLoad = [];
        end
    end
end