%% Print class
%
% This is an abstract super-class in the Object Oriented Programming (OOP)
% paradigm that generically specifies methods for print analysis model
% information and results in the <main.html LESM (Linear Elements
% Structure Model)> program.
%
% This super-class is abstract because one cannot instantiate objects from
% it, since it contains abstract methods that should be implemented in
% (derived) sub-classes.
%
% Essentially, this super-class defines abstract and public methods
% that print analysis model information and results in a text file.
%
% The abstract methods are the functions that should be implemented
% in a (derived) sub-class that deals with specific types of models,
% such as a planar frame, planar truss, or a grillage model.
%
%% Sub-classes
% In the current implementation of LESM program there are three sub-classes
% of this super-class:
%%%
% * <print_truss2d.html Print planar truss analysis model and results>.
% * <print_frame2d.html Print planar frame analysis model and results>.
% * <print_grillage.html Print grillage analysis model and results>.
%
%% Authors
% Luiz Fernando Martha and Rafael Lopez Rangel
%
%% History
% @version 1.00
%
% Initial version: August-December 2016
%%%
% Prepared for the course ENG 1240 - Analise de Estruturas III, 2016.2,
% Department of Civil Engineering, PUC-Rio.
%
classdef Print < hgsetget
    %%
    % <http://www.mathworks.com/help/matlab/handle-classes.html
    % See documentation on *hgsetget* handle super-class>.
    
    %% Public attributes
    properties (SetAccess = public, GetAccess = public)
        drv = [];   % analysis driver object
        txt = [];   % integer identifier of output file to write results
    end
    
    %% Constructor method
    methods
        %-----------------------------------------------------------------------
        function obj = Print(drv,txt)
            if (nargin > 0)
                obj.drv = drv;
                obj.txt = txt;
            end
        end
    end
    
    %% Public set methods
    methods
        %-----------------------------------------------------------------------
        % Set analysis driver object property.
        function obj = set.drv(obj,drv)
            obj.drv = drv;
        end
        
        %-----------------------------------------------------------------------
        % Set integer identifier of output file property.
        function obj = set.txt(obj,txt)
            obj.txt = txt;
        end
    end
    
    %% Public get methods
    methods
        %-----------------------------------------------------------------------
        % Get analysis driver object property.
        function drv = get.drv(obj)
            drv = obj.drv;
        end
        
        %-----------------------------------------------------------------------
        % Get integer identifier of output file property.
        function txt = get.txt(obj)
            txt = obj.txt;
        end
    end
    
    
    %% Abstract methods for textual results
    methods (Abstract)
        %-----------------------------------------------------------------------
        % Print analyis results in a text document.
        obj = results(obj)
        
        %-----------------------------------------------------------------------
        % Print analyis model type in a text document.
        obj = analysisLabel(obj)
        
        %-----------------------------------------------------------------------
        % Print nodal restraint conditions in a text document.
        obj = nodalSupport(obj)
        
        %-----------------------------------------------------------------------
        % Print nodal prescribed displacements in a text document.
        obj = nodalPrescDisp(obj)
        
        %-----------------------------------------------------------------------
        % Print nodal loads in a text document.
        obj = nodalLoads(obj)
        
        %-----------------------------------------------------------------------
        % Print elements information in a text document.
        obj = elements(obj)
        
        %-----------------------------------------------------------------------
        % Print element loads information in a text document.
        obj = elementLoads(obj)
        
        %-----------------------------------------------------------------------
        % Print nodal displacement/rotation in a text document.
        obj = nodalDisplRot(obj)
        
        %-----------------------------------------------------------------------
        % Print nodal force and moment support reactions in a text document.
        obj = reactions(obj)
        
        %-----------------------------------------------------------------------
        % Print internal forces at element nodes in a text document.
        obj = intForces(obj)
    end
    
    %% Public methods
    methods
        %-----------------------------------------------------------------------
        % Print header of analysis results in a text document.
        function obj = header(obj)
            fprintf( obj.txt, '\n======================================' );
            fprintf( obj.txt, '===================\n' );
            fprintf( obj.txt, ' PONTIFICIA UNIVERSIDADE CATOLICA DO' );
            fprintf( obj.txt, ' RIO DE JANEIRO\n' );
            fprintf( obj.txt, ' DEPARTAMENTO DE ENGENHARIA CIVIL\n' );
            fprintf( obj.txt, ' LESM - Linear Elements Structure Model' );
            fprintf( obj.txt, ' analysis program\n' );
            fprintf( obj.txt, '======================================' );
            fprintf( obj.txt, '===================\n' );
        end
        
        %-----------------------------------------------------------------------
        % Print global model description in a text document.
        function obj = modelDescrip(obj)
            % Calculate number of materials
            matid = zeros(1,obj.drv.nel);
            for e = 1:obj.drv.nel
                matid(e) = obj.drv.elems(e).material.id;
            end
            nmat = size(unique(matid),2);
            
            % Calculate number of cross sections
            secid = zeros(1,obj.drv.nel);
            for e = 1:obj.drv.nel
                secid(e) = obj.drv.elems(e).section.id;
            end
            nsec = size(unique(secid),2);
            
            % Calculate number of nodal loads
            n_nodalload = 0;
            for n = 1:obj.drv.nnp
                if size(obj.drv.nodes(n).nodalLoad,2) > 0
                    n_nodalload = n_nodalload + 1;
                end
            end
            
            % Calculate number of member uniform loads
            n_uniformload  = 0;
            for e = 1:obj.drv.nel
                if size(obj.drv.elems(e).uniformLoad,2) > 0
                    n_uniformload = n_uniformload + 1;
                end
            end
            
            % Calculate number of member linear loads
            n_linearload = 0;
            for e = 1:obj.drv.nel
                if size(obj.drv.elems(e).linearLoad,2) > 0
                    n_linearload = n_linearload + 1;
                end
            end
            
            % Calculate number of nodes with presc. disp.
            nnprescdispl = 0;
            for n = 1:obj.drv.nnp
                if size(obj.drv.nodes(n).prescDispl,2) > 0
                    nnprescdispl = nnprescdispl + 1;
                end
            end
            
            % Calculate number of members with temp. var.
            ntempvar = 0;
            for e = 1:obj.drv.nel
                if (obj.drv.elems(e).tempVar_GC ~= 0) || ...
                        (obj.drv.elems(e).tempVar_Y ~= 0)  || ...
                        (obj.drv.elems(e).tempVar_Z ~= 0)
                    ntempvar = ntempvar + 1;
                end
            end
            
            fprintf( obj.txt, '\n\n----------------------------------------\n' );
            fprintf( obj.txt, 'M O D E L  D E S C R I P T I O N:\n' );
            fprintf( obj.txt, '----------------------------------------\n' );
            fprintf( obj.txt, 'NUMBER OF NODES....................:%4d\n', obj.drv.nnp );
            fprintf( obj.txt, 'NUMBER OF ELEMENTS (MEMBERS).......:%4d\n', obj.drv.nel );
            fprintf( obj.txt, 'NUMBER OF DEGREES OF FREEDOM.......:%4d\n', obj.drv.neq );
            fprintf( obj.txt, 'NUMBER OF FREE DEGREES OF FREEDOM..:%4d\n', obj.drv.neqfree );
            fprintf( obj.txt, 'NUMBER OF FIXED DEGREES OF FREEDOM.:%4d\n', obj.drv.neqfixed );
            fprintf( obj.txt, 'NUMBER OF MATERIALS................:%4d\n', nmat );
            fprintf( obj.txt, 'NUMBER OF CROSS SECTION PROPERTIES.:%4d\n', nsec );
            fprintf( obj.txt, 'NUMBER OF NODAL LOADS..............:%4d\n', n_nodalload );
            fprintf( obj.txt, 'NUMBER OF MEMBER UNIFORM LOADS.....:%4d\n', n_uniformload );
            fprintf( obj.txt, 'NUMBER OF MEMBER LINEAR LOADS......:%4d\n', n_linearload );
            fprintf( obj.txt, 'NUMBER OF NODES WITH PRESC. DISPL..:%4d\n', nnprescdispl );
            fprintf( obj.txt, 'NUMBER OF MEMBERS WITH TEMP. VAR...:%4d\n', ntempvar );
            fprintf( obj.txt, '----------------------------------------\n' );
        end
        
        %-----------------------------------------------------------------------
        % Print nodal coordinates in a text document.
        function obj = nodalCoords(obj)
            fprintf( obj.txt, '\n\n--------------------------------\n' );
            fprintf( obj.txt, 'N O D A L  C O O R D I N A T E S \n' );
            fprintf( obj.txt, '--------------------------------\n' );
            fprintf( obj.txt, ' NODE     COORD X      COORD Y      COORD Z    \n' );
            
            for n = 1:obj.drv.nnp
                fprintf( obj.txt, '%4d   %10.3f   %10.3f   %10.3f\n', ...
                    n, obj.drv.nodes(n).coord_X, obj.drv.nodes(n).coord_Y, obj.drv.nodes(n).coord_Z );
            end
        end
        
        %-----------------------------------------------------------------------
        % Clean data structure of an Print object.
        function obj = clean(obj)
            obj.drv = [];
            obj.txt = [];
        end
    end
end