%% Print_Frame2D class
%
% This is a sub-class, in the Object Oriented Programming (OOP) paradigm,
% of super-class <print.html *Print*> in the <main.html LESM (Linear Elements
% Structure Model)> program. This sub-class implements abstract methods,
% defined in super-class *Print*, that deal with planar frame analysis model
% of linear structure elements.
%
%% Authors
% Luiz Fernando Martha and Rafael Lopez Rangel
%
%% History
% @version 1.00
%
% Initial version: August-December 2016
%%%
% Prepared for the course ENG 1240 - Analise de Estruturas III, 2016.2,
% Department of Civil Engineering, PUC-Rio.
%
classdef Print_Frame2D < Print
    %%
    % <print.html See documentation on *Print* super-class>.
    
    %% Constructor method
    methods
        %-----------------------------------------------------------------------
        function obj = Print_Frame2D()
            obj = obj@Print([],[]);
        end
    end
    
    %% Public methods
    % Implementation of the abstract methods declared in super-class <print.html *Print*>.
    methods
        %-----------------------------------------------------------------------
        % Print analyis results in a text document.
        function obj = results(obj)
            obj.header();
            obj.modelDescrip();
            obj.nodalCoords();
            obj.analysisLabel();
            obj.nodalSupport();
            obj.nodalPrescDisp();
            obj.nodalLoads();
            obj.elements();
            obj.elementLoads();
            obj.nodalDisplRot();
            obj.reactions();
            obj.intForces();
        end
        
        %-----------------------------------------------------------------------
        % Print analyis model type in a text document.
        function obj = analysisLabel(obj)
            fprintf( obj.txt, '\n\n----------------------------\n' );
            fprintf( obj.txt, 'ANALYSIS MODEL: ' );
            fprintf( obj.txt, 'PLANAR FRAME\n' );
            fprintf( obj.txt, '----------------------------\n' );
        end
        
        %-----------------------------------------------------------------------
        % Print nodal restraint conditions in a text document.
        function obj = nodalSupport(obj)
            fprintf( obj.txt, '\n\n------------------------------\n' );
            fprintf( obj.txt, 'N O D A L  R E S T R A I N T S \n' );
            fprintf( obj.txt, '------------------------------\n' );
            fprintf( obj.txt, ' NODE   DISPL X   DISPL Y   ROTAT Z \n' );
            
            for n = 1:obj.drv.nnp
                if(obj.drv.nodes(n).ebc(1) == 1)
                    node_restr1 = 'FIXED';
                else
                    node_restr1 = 'FREE';
                end
                
                if(obj.drv.nodes(n).ebc(2) == 1)
                    node_restr2 = 'FIXED';
                else
                    node_restr2 = 'FREE';
                end
                
                if(obj.drv.nodes(n).ebc(6) == 1)
                    node_restr3 = 'FIXED';
                else
                    node_restr3 = 'FREE';
                end
                
                fprintf( obj.txt, '%4d     %5s     %5s     %5s\n', n, node_restr1, node_restr2, node_restr3 );
            end
        end
        
        %-----------------------------------------------------------------------
        % Print nodal prescribed displacements in a text document.
        function obj = nodalPrescDisp(obj)
            fprintf( obj.txt, '\n\n--------------------------------\n' );
            fprintf( obj.txt, 'N O D A L  P R E S C.  D I S P L. \n' );
            fprintf( obj.txt, '--------------------------------\n' );
            fprintf( obj.txt, ' NODE     DX          DY          RZ    \n' );
            
            for n = 1:obj.drv.nnp
                if size(obj.drv.nodes(n).prescDispl,2) > 0
                    fprintf( obj.txt, '%4d %10.3f %10.3f %10.3f\n', ...
                        n, obj.drv.nodes(n).prescDispl(1), obj.drv.nodes(n).prescDispl(2), obj.drv.nodes(n).prescDispl(6) );
                end
            end
        end
        
        %-----------------------------------------------------------------------
        % Print nodal loads in a text document.
        function obj = nodalLoads(obj)
            fprintf( obj.txt, '\n\n--------------------------------\n' );
            fprintf( obj.txt, 'N O D A L  L O A D S \n' );
            fprintf( obj.txt, '--------------------------------\n' );
            fprintf( obj.txt, ' NODE     FX          FY          MZ    \n' );
            
            for n = 1:obj.drv.nnp
                if size(obj.drv.nodes(n).nodalLoad,2) > 0
                    fprintf( obj.txt, '%4d %10.3f %10.3f %10.3f\n', ...
                        n, obj.drv.nodes(n).nodalLoad(1), obj.drv.nodes(n).nodalLoad(2), obj.drv.nodes(n).nodalLoad(6) );
                end
            end
        end
        
        %-----------------------------------------------------------------------
        % Print elements information in a text document.
        function obj = elements(obj)
            include_constants;
            
            fprintf( obj.txt, '\n\n---------------\n' );
            fprintf( obj.txt, 'E L E M E N T S\n' );
            fprintf( obj.txt, '---------------\n' );
            fprintf( obj.txt, ' ELEM  TYPE           MAT  SEC  HINGEi HINGEf  NODEi NODEf\n' );
            
            for e = 1:obj.drv.nel
                element_type = obj.drv.elems(e).type;
                
                switch element_type
                    case MEMBER_TRUSS
                        type = 'Truss member ';
                    case MEMBER_NAVIER
                        type = 'Navier member';
                    case MEMBER_TIMOSHENKO
                        type = 'Timoshenko mb';
                end
                
                if( obj.drv.elems(e).hingei == HINGED_END )
                    hingei = 'yes';
                else
                    hingei = 'no ';
                end
                
                if( obj.drv.elems(e).hingef == HINGED_END )
                    hingef = 'yes';
                else
                    hingef = 'no ';
                end
                
                mat = obj.drv.elems(e).material.id;
                sec = obj.drv.elems(e).section.id;
                nodei = obj.drv.elems(e).nodei.id;
                nodef = obj.drv.elems(e).nodef.id;
                
                fprintf( obj.txt, '%4d   %13s %4d %4d   %3s    %3s    %4d  %4d\n', ...
                    e, type, mat, sec, hingei, hingef, nodei, nodef );
            end
        end
        
        %-----------------------------------------------------------------------
        % Print element loads information in a text document.
        function obj = elementLoads(obj)
            fprintf( obj.txt, '\n\n------------------------\n' );
            fprintf( obj.txt, 'E L E M E N T  L O A D S\n' );
            fprintf( obj.txt, '------------------------\n' );
            fprintf( obj.txt, ' ELEM  UNIF_DIR.   UNIF_X     UNIF_Y    LIN_DIR.    LIN_Xi     LIN_Yi     LIN_Xj     LIN_Yj      DTX        DTY\n' );
            
            for e = 1:obj.drv.nel
                if obj.drv.elems(e).uniformDirection == 0
                    unifdir = 'GLOBAL';
                else
                    unifdir = 'LOCAL';
                end
                
                if obj.drv.elems(e).linearDirection == 0
                    lindir = 'GLOBAL';
                else
                    lindir = 'LOCAL ';
                end
                
                if size(obj.drv.elems(e).uniformLoad,2) > 0
                    unifx = obj.drv.elems(e).uniformLoad(1);
                    unify = obj.drv.elems(e).uniformLoad(2);
                else
                    unifx = 0.0;
                    unify = 0.0;
                end
                
                if size(obj.drv.elems(e).linearLoad,2) > 0
                    linxi = obj.drv.elems(e).linearLoad(1);
                    linyi = obj.drv.elems(e).linearLoad(2);
                    linxj = obj.drv.elems(e).linearLoad(4);
                    linyj = obj.drv.elems(e).linearLoad(5);
                else
                    linxi = 0.0;
                    linyi = 0.0;
                    linxj = 0.0;
                    linyj = 0.0;
                end
                
                dtx = obj.drv.elems(e).tempVar_GC;
                dty = obj.drv.elems(e).tempVar_Y;
                
                fprintf( obj.txt, '%4d   %6s %10.2f %10.2f     %6s %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n', ...
                    e, unifdir, unifx, unify, lindir, linxi, linyi, linxj, linyj, dtx, dty );
            end
        end
        
        %-----------------------------------------------------------------------
        % Print nodal displacement/rotation in a text document.
        function obj = nodalDisplRot(obj)
            fprintf( obj.txt, '\n\n-------------------------------\n' );
            fprintf( obj.txt, 'NODAL DISPLACEMENTS / ROTATIONS \n' );
            fprintf( obj.txt, '-------------------------------\n' );
            fprintf( obj.txt, ' NODE       DISPL X        DISPL Y        ROTAT Z\n' );
            
            for n = 1:obj.drv.nnp
                dx = obj.drv.D(obj.drv.ID(1,n));
                dy = obj.drv.D(obj.drv.ID(2,n));
                rz = obj.drv.D(obj.drv.ID(3,n));
                fprintf( obj.txt, '%4d     %10.5f     %10.5f     %10.5f\n', n, dx, dy, rz );
            end
        end
        
        %-----------------------------------------------------------------------
        % Print nodal force and moment support reactions in a text document.
        function obj = reactions(obj)
            fprintf( obj.txt, '\n\n-----------------------\n' );
            fprintf( obj.txt, 'NODAL SUPPORT REACTIONS\n' );
            fprintf( obj.txt, '-----------------------\n' );
            fprintf( obj.txt, ' NODE       FORCE X      FORCE Y      MOMENT Z\n' );
            
            for n = 1:obj.drv.nnp
                if( (obj.drv.ID(1,n) > obj.drv.neqfree) || ...
                    (obj.drv.ID(2,n) > obj.drv.neqfree) || ...
                    (obj.drv.ID(3,n) > obj.drv.neqfree) )
                    if(obj.drv.ID(1,n) > obj.drv.neqfree)
                        node_reaction1 = obj.drv.F(obj.drv.ID(1,n));
                    else
                        node_reaction1 = 0.0;
                    end
                    
                    if(obj.drv.ID(2,n) > obj.drv.neqfree)
                        node_reaction2 = obj.drv.F(obj.drv.ID(2,n));
                    else
                        node_reaction2 = 0.0;
                    end
                    
                    if(obj.drv.ID(3,n) > obj.drv.neqfree)
                        node_reaction3 = obj.drv.F(obj.drv.ID(3,n));
                    else
                        node_reaction3 = 0.0;
                    end
                    
                    fprintf( obj.txt, '%4d     %10.3f   %10.3f   %10.3f\n', n, ...
                        node_reaction1, node_reaction2, node_reaction3 );
                end
            end
        end
        
        %-----------------------------------------------------------------------
        % Print internal forces at element nodes in a text document.
        function obj = intForces(obj)
            fprintf( obj.txt, '\n\n--------------------------------\n' );
            fprintf( obj.txt, 'INTERNAL FORCES AT ELEMENT NODES\n' );
            fprintf( obj.txt, '--------------------------------\n' );
            fprintf( obj.txt, ' ELEM      AXIAL FORCE             SHEAR FORCE             BENDING MOMENT\n' );
            fprintf( obj.txt, '           Nodei      Nodef        Nodei      Nodef        Nodei      Nodef\n' );
            
            for e = 1:obj.drv.nel
                fprintf( obj.txt, '%4d   %10.3f %10.3f   %10.3f %10.3f   %10.3f %10.3f\n', e, ...
                    obj.drv.elems(e).axial_force(1), obj.drv.elems(e).axial_force(2), ...
                    obj.drv.elems(e).shear_force_Y(1), obj.drv.elems(e).shear_force_Y(2), ...
                    obj.drv.elems(e).bending_moment_Z(1), obj.drv.elems(e).bending_moment_Z(2) );
            end
        end
    end
end