%% LESM - Linear Elements Structure Model
%
% This is the main driver file of LESM.  This is a MATLAB program for
% linear-elastic, displacement-based, linear elements structure model
% analysis, using the direct stiffness method.
% The program creates a structure model data, assembles a system of equations,
% solves the system and prints out the response data.
%
%% Authors
% Luiz Fernando Martha and Rafael Lopez Rangel
%
% Pontifical Catholic University of Rio de Janeiro - PUC-Rio
%
% Department of Civil Engineering and Tecgraf Institute of Technical-Scientific
% Software Development of PUC-Rio (Tecgraf/PUC-Rio)
%
%% Analysis model types
% LESM considers only linear elastic structural analysis, and currently
% three types of structural models:
%%%
% *Truss 2D analysis*
%
% A truss model is a common form of analysis model, with the
% following assumptions:
%%% 
% * Truss members are bars connected at their ends only, and they are
%   connected by friction-less pins. Therefore, a truss member does
%   not present any secondary bending moment induced by rotation
%   continuity at joints.
% * A truss model is loaded only at joints, which are also called
%   nodes. Any load action along a member, such as self weight, is
%   statically transferred as concentrated forces to the member end
%   nodes.
% * Local bending of members due to transversal member loads are
%   neglected, when compared to the effect of global load acting on
%   the truss.
% * Therefore, there is only one type of internal force in a truss
%   member: axial internal force, which may be tension or
%   compression.
% * As truss members have (by assumption) only axial behavior, the
%   area of its cross-section is the single geometric parameter
%   needed.
% * In the current implementation of LESM, only 2D truss models are
%   considered. A model is laid on the XY plane.
% * Each node of a 2D truss model has two d.o.f.'s (degrees of
%   freedom): a horizontal displacement in X direction and a vertical
%   displacement in Y direction.
%%%
% *Frame 2D analysis*
%
% A frame model is also made up of bars, which are called beams
% (horizontal bars) or columns (vertical bars). Inclined members
% are also called beams. In the present context, these types of bars
% are generically called "frame members". A continuous beam (an
% assemblage of connected beams) is considered as a frame model by
% LESM.  These are the basic assumptions of a frame model:
%%%
% * Frame members are usually rigidly connected at the joints.
%   However, a frame member might have a hinge (rotation liberation)
%   at an end or hinges at both ends. A frame member with hinges at
%   both end works like a truss member.  A truss model could be seen
%   a frame model with complete hinges at both its nodes.
% * In the current implementation of LESM, only 2D frame models are
%   considered. A model is laid on the XY plane.
% * By assumption, there is only in-plane behavior, that is, there is
%   no displacement transversal to the frame plane.
% * Internal forces at any cross-section of a plane frame member are:
%   axial force, shear force, and bending moment.
% * The geometric parameters of a plane frame member cross-section
%   are its full area (for axial behavior), its effective shear area,
%   and its moment of inertia (for bending behavior).
% * Each node of a 2D frame model has three d.o.f.'s: a horizontal
%   displacement in X direction, a vertical displacement in Y
%   direction, and a rotation about the Z axis.
%%%
% *Grillage analysis*
%
% A grillage model is a common form of analysis model for building
% stories and bridge decks. Its key features are:
% It is a 2D model, which in LESM is considered on the XY plane.
%%%
% * Beam members are laid out in a grid pattern in a single plane,
%   rigidly connected at nodes. However, a grillage member might have
%   a hinge (rotation liberation) at an end or hinges at both ends.
%   In LESM, it is assumed that a hinge in a grillage member releases
%   continuity of both bending and torsion rotations.
% * By assumption, there is only out-of-plane behavior, which
%   includes displacement transversal to the grillage plane, and
%   rotations about axes on the plane.
% * Internal forces at any cross-section of a grillage member are:
%   shear force (in the plane transversal direction), bending moment
%   (bending of members occurs in a plane perpendicular to the
%   grillage plane), and torsion moment (about a member axis).
%   By assumption, there is no axial force in a grillage member.
% * The geometric parameters of a grillage member cross-section are
%   its effective shear area, its moment of inertia for bending
%   behavior, and its moment of inertia for torsion behavior.
% * Each node of a grillage model has three d.o.f.'s: a transversal
%   displacement in Z direction, a rotation about the plane X axis,
%   and a rotation about the plane Y axis.
%
%% Structural element types
%
% In LESM, structural elements are generically called members, bars,
% or beam elements. For frame or grillage models, whose members have
% bending effects, two types of beam elements are considered:
%
% * Navier (Euler-Bernoully) beam element
% * Timoshenko beam element
%
% In truss models, the two types of beam elements may be used
% indistinguishably, since there is no bending behavior of a member,
% and Euler-Bernoully beams and Timoshenko beams are equivalent for
% the axial behavior.
%
%% Load types
%
% There are four types of loads:
%
% * Concentrated nodal load in global axis directions.
% * Uniformely distributed force on members, spanning its entire
%   length, in local or in global axis directions.
% * Linearly distributed force on members, spanning its entire length,
%   in local or in global axis directions.
% * Uniform temperature variation on faces of members, resulting on
%   axial uniform temperature variation and uniform temperature
%   gradient.
%
% In addition, nodal prescribed displacement and rotations may be
% specified.
%
%% Components of concentrated nodal loads
% * In truss 2D models, concentrated nodal loads are force components
%   in global X and global Y directions.
% * In frame 2D models, concentrated nodal loads are force components
%   in global X and global Y directions, and a moment component
%   about global Z direction.
% * In grillage models, concentrated nodal loads are one force
%   component in global Z direction (transversal to the model plane),a
%   and two moment components about global X and global Y axes.
%
%% Local axes of a member
%
% In the 2D models of LESM, the local axes of a member are defined
% uniquely for all the types of models in the following manner:
%
% * The local z-axis of a member is always in the direction of the
%   global Z-axis, which is perpendicular to the model plane.
% * The local x-axis of a member is its longitudinal axis, in the
%   direction from its first node to its final node.
% * The local y-axis of a member is perpendicular to the longitudinal
%   member x-axis in such a way that the cross-product
%   (x-axis * y-axis) results in a vector in the global Z direction.
%
%% Components of distributed forces on members
% * In truss 2D or frame 2D models, the uniformely or linearly
%   distributed force has two components, which are in global X or Y
%   axes, or in local x or y axes directions.
% * In grillage models, the uniformely or linearly distributed force
%   has only one component, which is in global Z direction.
%
%% Load cases
%
% It is assumed that there is a single load case.
%
%% Object oriented classes
% This program adopts an Object Oriented Programming (OOP) paradigm, in
% which the following OOP classes are implemented:
%%%
% * <drv.html Drv: analysis driver class>.
% * <anm.html Anm: analysis model class>.
% * <node.html Node: node class>.
% * <elem.html Elem: element class>.
% * <material.html Material: material class>.
% * <section.html Section: cross-section class>.
% * <print.html Print: print class>.
%
%% Auxiliary functions and files
%%%
% * <preframe2dtest.html preFrame2dTest: File with planar frame test model>.
% * <include_constants.html include_constants: File with global constants>.
%
%% History
% @version 1.00
%
% Initial version: August-December 2016
%%%
% Prepared for the course ENG 1240 - Analise de Estruturas III, 2016.2,
% Department of Civil Engineering, PUC-Rio.
%

clc;
clear all;
close all;

% Preprocessing
fprintf(1,'Pre-processing...\n');
[drv,print] = preFrame2dTest;
%[drv,print] = preTruss2dTest;

% Process the provided model data according to the direct stiffness method
status = drv.analyze();

% Print on default output analysis results
if(status == 1)
    fprintf(1,'Printing results...\n');
    print.results();
end

fprintf(1,'\n\nFinished.\n');