%% preFrame2dTest - preprocessor file of LESM
%
% This is a function that creates a test model in program <main.html
% *LESM*> - Linear Elements Structure Model.  This is a MATLAB program
% for linear-elastic, displacement-based, linear elements structure
% model analysis, according to the direct stiffness method.
% The program assembles global equilibrium system of equations,
% calculates nodal displacements and rotations, and computes
% reactions and elements internal forces.
%
% The test model created in this file is a planar frame specified in
% Ftool file frame_test_a.ftl, using Navier (Euler-Bernoulli) members.
%
%% Authors
% Luiz Fernando Martha and Rafael Lopez Rangel
%
% Pontifical Catholic University of Rio de Janeiro - PUC-Rio
%
% Department of Civil Engineering and Tecgraf Institute of Technical-Scientific
% Software Development of PUC-Rio (Tecgraf/PUC-Rio)
%
%% History
% @version 1.00
%
% Initial version: August-December 2016
%%%
% Prepared for the course ENG 1240 - Analise de Estruturas III, 2016.2,
% Department of Civil Engineering, PUC-Rio.
%

function [drv,print] = preFrame2dTest()
% Preprocess input data and initialize global arrays.
% Output:
%  drv: handle to analysis driver object
%  print: handle to analysis print object

    include_constants;

    % Create an analysis driver
    drv = Drv();

    % Create an analyis model
    drv.anm = Anm_Frame2D();
    
    % Create print object;
    print = Print_Frame2D();
    print.drv = drv;
    print.txt = 1;
    
    % Set number of nodes and create array of nodes
    drv.nnp = 4;
    nodes(1,drv.nnp) = Node();
    drv.nodes = nodes;

    % Compute total number of equations of model
    ndof = drv.anm.ndof;  % number of degrees-of-freedom per node
    drv.neq = drv.nnp * ndof;

    % Set node ids and coordinates
    drv.nodes(1).id = 1;
    drv.nodes(1).coord_X =  0.0;
    drv.nodes(1).coord_Y =  0.0;
    drv.nodes(1).coord_Z =  0.0;
    drv.nodes(2).id = 2;
    drv.nodes(2).coord_X = 14.0;
    drv.nodes(2).coord_Y =  0.0;
    drv.nodes(2).coord_Z =  0.0;
    drv.nodes(3).id = 3;
    drv.nodes(3).coord_X =  0.0;
    drv.nodes(3).coord_Y = 10.0;
    drv.nodes(3).coord_Z =  0.0;
    drv.nodes(4).id = 4;
    drv.nodes(4).coord_X = 14.0;
    drv.nodes(4).coord_Y = 12.0;
    drv.nodes(4).coord_Z =  0.0;
    
    % Set node (support) essential boundary conditions (ebc)
    drv.nodes(1).ebc = [1 1 1 1 1 1];
    drv.nodes(2).ebc = [1 1 1 1 1 0];
    drv.nodes(3).ebc = [0 0 1 1 1 0];
    drv.nodes(4).ebc = [0 0 1 1 1 0];

    % Create one material and set its properties
    nmat = 1;
    materials(1,nmat) = Material();
    materials(1).id = 1;
    materials(1).elasticity = 2e+7;
    materials(1).poisson = 0.3;
    materials(1).thermexp = 1e-5;
    
    % Create one cross-section and set its properties
    nsec = 1;
    sections(1,nsec) = Section();
    sections(1).id = 1;
    sections(1).area_X = 0.05;
    sections(1).area_Y = 0.04;
    sections(1).area_Z = 0.0;
    sections(1).inertia_X = 0.0;
    sections(1).inertia_Y = 0.0;
    sections(1).inertia_Z = 0.0025;
    sections(1).cw = 0.0;
    sections(1).height_Y = 0.6;
    sections(1).height_Z = 0.0;
    
    % Set number of elements and create array of elements
    drv.nel = 3;
    elems(1,drv.nel) = Elem_Navier();
    drv.elems = elems;

    % Set element analysis model, element type, element material and
    % element cross-section
    for i = 1:drv.nel
        drv.elems(i).anm = drv.anm;
        drv.elems(i).type = MEMBER_NAVIER;
        drv.elems(i).material = materials(1);
        drv.elems(i).section = sections(1);
    end
    
    % Set elements nodal incidence
    drv.elems(1).nodei = drv.nodes(1);
    drv.elems(1).nodef = drv.nodes(3);
    drv.elems(2).nodei = drv.nodes(3);
    drv.elems(2).nodef = drv.nodes(4);
    drv.elems(3).nodei = drv.nodes(4);
    drv.elems(3).nodef = drv.nodes(2);
    
    % Set element end hinges
    drv.elems(1).hingei = CONTINUOUS_END;
    drv.elems(1).hingef = CONTINUOUS_END;
    drv.elems(2).hingei = CONTINUOUS_END;
    drv.elems(2).hingef = HINGED_END;
    drv.elems(3).hingei = CONTINUOUS_END;
    drv.elems(3).hingef = CONTINUOUS_END;
    
    % Calculate element lengths, orientation cosines, and
    % rotation matrices
    for i = 1:drv.nel
        % Get projection of element length in X, Y, and Z axis
        dx = drv.elems(i).nodef.coord_X - drv.elems(i).nodei.coord_X;
        dy = drv.elems(i).nodef.coord_Y - drv.elems(i).nodei.coord_Y;
        dz = drv.elems(i).nodef.coord_Z - drv.elems(i).nodei.coord_Z;

        % Calculate element length
        L = sqrt(dx^2 + dy^2 + dz^2);
        drv.elems(i).length = L;

        % Calculate element orientation cosines
        cx = dx/L;
        cy = dy/L;
        cz = dz/L;
        drv.elems(i).cosine_X = cx;
        drv.elems(i).cosine_Y = cy;
        drv.elems(i).cosine_Z = cz;

        % Set auxiliary vector in local XZ plane in global Z direction
        drv.elems(i).vz_X = 0;
        drv.elems(i).vz_Y = 0;
        drv.elems(i).vz_Z = 1;
        
        % Get element rotation transformation matrix from global system to local system
        rot = drv.anm.gblToLocElemRotMtx(drv.elems(i));
        drv.elems(i).rot = rot;
    end

    % Set node prescribed displacements and rotations
    drv.nodes(1).prescDispl = [0  -0.03  0  0  0  0.02];
    
    % Set nodal concentrated loads
    drv.nodes(3).nodalLoad = [20  0  0  0  0  0];
    
    % Set element uniformely distributed loads
    drv.elems(2).uniformDirection = LOCAL_LOAD;
    drv.elems(2).uniformLoad = [0  -10  0];
    
    % Set element linearly distributed loads
    drv.elems(1).linearDirection = GLOBAL_LOAD;
    drv.elems(1).linearLoad = [20  0  0  10  0  0];
    
end